# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from kaskada.kaskada.v1alpha import view_service_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2


class ViewServiceStub(object):
    """A service for managing Views.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListViews = channel.unary_unary(
                '/kaskada.kaskada.v1alpha.ViewService/ListViews',
                request_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.ListViewsRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.ListViewsResponse.FromString,
                )
        self.GetView = channel.unary_unary(
                '/kaskada.kaskada.v1alpha.ViewService/GetView',
                request_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.GetViewRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.GetViewResponse.FromString,
                )
        self.CreateView = channel.unary_unary(
                '/kaskada.kaskada.v1alpha.ViewService/CreateView',
                request_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.CreateViewRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.CreateViewResponse.FromString,
                )
        self.DeleteView = channel.unary_unary(
                '/kaskada.kaskada.v1alpha.ViewService/DeleteView',
                request_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.DeleteViewRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.DeleteViewResponse.FromString,
                )


class ViewServiceServicer(object):
    """A service for managing Views.
    """

    def ListViews(self, request, context):
        """Lists views.

        The response includes all Views defined for the Kaskada account.
        If a search string is provided, only Views matching the search string
        are returned.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetView(self, request, context):
        """Gets a view.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateView(self, request, context):
        """Creates a view.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteView(self, request, context):
        """Deletes a view.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ViewServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListViews': grpc.unary_unary_rpc_method_handler(
                    servicer.ListViews,
                    request_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.ListViewsRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.ListViewsResponse.SerializeToString,
            ),
            'GetView': grpc.unary_unary_rpc_method_handler(
                    servicer.GetView,
                    request_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.GetViewRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.GetViewResponse.SerializeToString,
            ),
            'CreateView': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateView,
                    request_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.CreateViewRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.CreateViewResponse.SerializeToString,
            ),
            'DeleteView': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteView,
                    request_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.DeleteViewRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.DeleteViewResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'kaskada.kaskada.v1alpha.ViewService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ViewService(object):
    """A service for managing Views.
    """

    @staticmethod
    def ListViews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v1alpha.ViewService/ListViews',
            kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.ListViewsRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.ListViewsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v1alpha.ViewService/GetView',
            kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.GetViewRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.GetViewResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v1alpha.ViewService/CreateView',
            kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.CreateViewRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.CreateViewResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteView(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v1alpha.ViewService/DeleteView',
            kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.DeleteViewRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v1alpha_dot_view__service__pb2.DeleteViewResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from kaskada.kaskada.v1alpha import file_service_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2


class FileServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetMetadata = channel.unary_unary(
                '/kaskada.kaskada.v1alpha.FileService/GetMetadata',
                request_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.GetMetadataRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.GetMetadataResponse.FromString,
                )
        self.MergeMetadata = channel.unary_unary(
                '/kaskada.kaskada.v1alpha.FileService/MergeMetadata',
                request_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.MergeMetadataRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.MergeMetadataResponse.FromString,
                )


class FileServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetMetadata(self, request, context):
        """Fetches the metadata.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def MergeMetadata(self, request, context):
        """Incorporate additional files into the table schema.

        This is initially planned to perform a *widening* merge. Specifically, this
        should be used when the table schema is "all (compatible) columns in any of
        the files", with `null` values when the file doesn't contain that column.

        We may add a separate method for checking file schemas against a fixed
        table schema, or make that a property of the merge request.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_FileServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.GetMetadata,
                    request_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.GetMetadataRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.GetMetadataResponse.SerializeToString,
            ),
            'MergeMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.MergeMetadata,
                    request_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.MergeMetadataRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.MergeMetadataResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'kaskada.kaskada.v1alpha.FileService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class FileService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v1alpha.FileService/GetMetadata',
            kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.GetMetadataRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.GetMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def MergeMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v1alpha.FileService/MergeMetadata',
            kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.MergeMetadataRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v1alpha_dot_file__service__pb2.MergeMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import operator
import pprint

from domonic.html import pre, table, td, th, tr
from domonic.utils import Utils

import kaskada.kaskada.v1alpha.view_service_pb2 as view_pb

from .formatters_helpers import (
    appendChildren,
    appendHtmlObjTableRowIfAttrExists,
    convert_fenl_datatype,
    get_datetime,
    get_properties,
    get_schema_dataframe,
    html_obj_table_row,
    html_table_row,
)

"""
NOTES:

Use `hasattr(obj, prop) to test if an object has a property`
Use `obj.HasField(prop) to test if an object has a specific one-of`
"""


def appendTimes(parent, obj):
    if hasattr(obj, "create_time"):
        create_time = get_datetime(getattr(obj, "create_time")).isoformat()
        parent.appendChild(html_obj_table_row("create_time", create_time))
    if hasattr(obj, "update_time"):
        update_time = get_datetime(getattr(obj, "update_time")).isoformat()
        parent.appendChild(html_obj_table_row("update_time", update_time))


def get_analysis_and_diagnostics_tables(obj):
    can_execute = (
        obj.analysis.can_execute if hasattr(obj, "analysis") else obj.can_execute
    )
    details = table(
        html_obj_table_row("can_execute", str(can_execute)), _class="kda_table"
    )
    diagnostics = None
    has_errors = False
    if (
        hasattr(obj, "fenl_diagnostics")
        and len(obj.fenl_diagnostics.fenl_diagnostics) > 0
    ):
        details.appendChild(
            html_obj_table_row("fenl_diagnostics", "(see Diagnostics tab)")
        )
        diagnostics, has_errors = get_fenl_diagnostics_html_and_has_errors(
            obj.fenl_diagnostics
        )
    return can_execute, has_errors, details, diagnostics


def get_fenl_diagnostics_html_and_has_errors(obj):
    diagnostics = table(_class="kda_table")
    for i in range(len(obj.fenl_diagnostics)):
        fenl_diag = obj.fenl_diagnostics[i]
        diagnostics.appendChild(html_obj_table_row(i, fenl_diag.formatted))
    return diagnostics, obj.num_errors > 0


def get_generic_object_html_and_schema_df(obj):
    props = get_properties(obj)
    obj_type = type(obj).__name__.lower()

    # try to pop off obj_id and obj_name first
    obj_name_key = f"{obj_type}_name"
    obj_name = props.pop(obj_name_key, None)

    # also remove create & update time from props
    props.pop("create_time", None)
    props.pop("update_time", None)

    rows = [
        # first row is item name
        html_obj_table_row(obj_name_key, obj_name),
        # stop showing item ids since they aren't actionable by the user
        # html_obj_table_row(obj_id_key, obj_id),
    ]

    # add remainder of fields
    for key in props:
        rows.append(html_obj_table_row(key, props[key]))

    schema_df = get_schema_dataframe(obj)
    details = table(_class="kda_table")
    appendChildren(details, rows)
    if schema_df is not None:
        details.appendChild(html_obj_table_row("schema", "(see Schema tab)"))
    appendTimes(details, obj)
    return details, schema_df


def get_materialization_html_and_schema_df(obj):
    schema_df = get_schema_dataframe(obj)
    details = table(_class="kda_table")
    appendHtmlObjTableRowIfAttrExists(details, obj, "materialization_name")
    appendHtmlObjTableRowIfAttrExists(details, obj, "query")

    if hasattr(obj, "destination") and obj.HasField("destination"):
        destination = table(_class="kda_table")
        details.appendChild(html_table_row("destination", destination))
        if hasattr(obj.destination, "redis_a_i") and obj.destination.HasField(
            "redis_a_i"
        ):
            redis_a_i = table(_class="kda_table")
            appendHtmlObjTableRowIfAttrExists(
                redis_a_i, obj.destination.redis_a_i, "host"
            )
            appendHtmlObjTableRowIfAttrExists(
                redis_a_i, obj.destination.redis_a_i, "port"
            )
            appendHtmlObjTableRowIfAttrExists(
                redis_a_i, obj.destination.redis_a_i, "db"
            )
            destination.appendChild(html_table_row("redis_a_i", redis_a_i))

    if hasattr(obj, "slice"):
        details.appendChild(html_table_row("slice", get_slice_request_html(obj.slice)))

    if schema_df is not None:
        details.appendChild(html_obj_table_row("schema", "(see Schema tab)"))

    appendTimes(details, obj)
    return details, schema_df


def get_metrics_html(obj):
    metrics = table(_class="kda_table")

    time_preparing = (
        obj.time_preparing.ToMilliseconds() if hasattr(obj, "time_preparing") else 0
    )
    metrics.appendChild(html_table_row("time_preparing", f"{time_preparing / 1000}s"))

    time_computing = (
        obj.time_computing.ToMilliseconds() if hasattr(obj, "time_computing") else 0
    )
    metrics.appendChild(html_table_row("time_computing", f"{time_computing / 1000}s"))

    metrics.appendChild(html_table_row("output_files", obj.output_files))
    return metrics


def get_raw_html(obj):
    return pre(pprint.pformat(obj).replace("\\n", ""))


def get_request_details_table_row_if_exists(obj):
    if hasattr(obj, "request_details") and hasattr(obj.request_details, "request_id"):
        nested_table = table(
            html_obj_table_row("request_id", obj.request_details.request_id),
            _class="kda_table",
        )
        return html_table_row("request_details", nested_table)
    return None


def get_result_type_and_schema_df(obj):
    result_type = Utils.escape(convert_fenl_datatype(obj))
    schema_df = None
    if obj.HasField("struct"):
        schema_df = get_schema_dataframe(obj.struct)
        if schema_df is not None:
            result_type += " (see Schema tab)"
    return result_type, schema_df


def get_slice_request_html(obj):
    details = table(_class="kda_table")
    if obj is not None and obj.HasField("percent"):
        details.appendChild(html_obj_table_row("percent", obj.percent.percent))
    else:
        details.appendChild(html_obj_table_row("None", "(full dataset used for query)"))
    return details


def get_table_html_and_schema_df(obj):
    schema_df = get_schema_dataframe(obj)
    details = table(_class="kda_table")
    appendHtmlObjTableRowIfAttrExists(details, obj, "table_name")
    appendHtmlObjTableRowIfAttrExists(details, obj, "entity_key_column_name")
    appendHtmlObjTableRowIfAttrExists(details, obj, "time_column_name")
    appendHtmlObjTableRowIfAttrExists(details, obj, "grouping_id")
    if obj.HasField("subsort_column_name"):
        details.appendChild(
            html_obj_table_row("subsort_column_name", obj.subsort_column_name.value)
        )

    if hasattr(obj, "table_source") and obj.HasField("table_source"):
        if hasattr(obj.table_source, "iceberg") and obj.table_source.HasField(
            "iceberg"
        ):
            iceberg = table(_class="kda_table")
            appendHtmlObjTableRowIfAttrExists(
                iceberg, obj.table_source.iceberg, "table_name"
            )
            appendHtmlObjTableRowIfAttrExists(
                iceberg, obj.table_source.iceberg, "namespace"
            )
            if hasattr(
                obj.table_source.iceberg, "config"
            ) and obj.table_source.iceberg.HasField("config"):
                config = table(_class="kda_table")
                if hasattr(
                    obj.table_source.iceberg.config, "glue"
                ) and obj.table_source.iceberg.config.HasField("glue"):
                    glue = table(_class="kda_table")
                    if hasattr(
                        obj.table_source.iceberg.config.glue, "warehouse"
                    ) and obj.table_source.iceberg.config.glue.HasField("warehouse"):
                        warehouse = table(_class="kda_table")
                        appendHtmlObjTableRowIfAttrExists(
                            warehouse,
                            obj.table_source.iceberg.config.glue.warehouse,
                            "bucket",
                        )
                        appendHtmlObjTableRowIfAttrExists(
                            warehouse,
                            obj.table_source.iceberg.config.glue.warehouse,
                            "region",
                        )
                        appendHtmlObjTableRowIfAttrExists(
                            warehouse,
                            obj.table_source.iceberg.config.glue.warehouse,
                            "prefix",
                        )
                        glue.appendChild(html_table_row("warehouse", warehouse))
                    config.appendChild(html_table_row("glue", glue))
                iceberg.appendChild(html_table_row("config", config))
            details.appendChild(html_table_row("iceberg", iceberg))

    appendHtmlObjTableRowIfAttrExists(details, obj, "version")
    if schema_df is not None:
        details.appendChild(html_obj_table_row("schema", "(see Schema tab)"))
    appendTimes(details, obj)
    return details, schema_df


def get_query_html(obj):
    details = table(_class="kda_table")
    appendHtmlObjTableRowIfAttrExists(details, obj, "query_id")
    details.appendChild(html_table_row("create_time", obj.create_time.ToJsonString()))
    appendHtmlObjTableRowIfAttrExists(details, obj, "expression")
    if hasattr(obj, "data_token_id"):
        details.appendChild(html_table_row("data_token", obj.data_token_id.value))
    if hasattr(obj, "views"):
        views = table(_class="kda_table")
        views.appendChild(tr(th("name"), th("expression")))
        obj.views.sort(key=operator.attrgetter("view_name"))
        for view in obj.views:
            views.appendChild(tr(td(view.view_name), td(view.expression)))
        details.appendChild(html_table_row("views", views))
    if hasattr(obj, "result_behavior"):
        result_behavior = "Unspecified"
        if obj.result_behavior == 1:
            result_behavior = "All Results"
        elif obj.result_behavior == 2:
            result_behavior = "Final Results"
        elif obj.result_behavior == 3:
            result_behavior = "Final Results at Time"
        details.appendChild(html_table_row("result_behavior", result_behavior))
    if hasattr(obj, "slice"):
        details.appendChild(html_table_row("slice", obj.slice))
    if hasattr(obj, "limits"):
        details.appendChild(html_table_row("limits", obj.limits))

    return details


def get_view_html_and_schema_df(obj: view_pb.View):
    details = table(_class="kda_table")
    appendHtmlObjTableRowIfAttrExists(details, obj, "view_name")
    appendHtmlObjTableRowIfAttrExists(details, obj, "expression")
    schema_df = None
    result_type, schema_df = get_result_type_and_schema_df(obj.result_type)
    if result_type is not None and result_type != "":
        details.appendChild(html_obj_table_row("result_type", result_type))

    appendTimes(details, obj)
    return details, schema_df



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from kaskada.kaskada.v1alpha import preparation_service_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2


class PreparationServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.PrepareData = channel.unary_unary(
                '/kaskada.kaskada.v1alpha.PreparationService/PrepareData',
                request_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.PrepareDataRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.PrepareDataResponse.FromString,
                )
        self.GetCurrentPrepID = channel.unary_unary(
                '/kaskada.kaskada.v1alpha.PreparationService/GetCurrentPrepID',
                request_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.GetCurrentPrepIDRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.GetCurrentPrepIDResponse.FromString,
                )


class PreparationServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def PrepareData(self, request, context):
        """Prepares the data from source files
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCurrentPrepID(self, request, context):
        """Returns the current preparation ID of the preparation (currently hard coded)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PreparationServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'PrepareData': grpc.unary_unary_rpc_method_handler(
                    servicer.PrepareData,
                    request_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.PrepareDataRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.PrepareDataResponse.SerializeToString,
            ),
            'GetCurrentPrepID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCurrentPrepID,
                    request_deserializer=kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.GetCurrentPrepIDRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.GetCurrentPrepIDResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'kaskada.kaskada.v1alpha.PreparationService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PreparationService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def PrepareData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v1alpha.PreparationService/PrepareData',
            kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.PrepareDataRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.PrepareDataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCurrentPrepID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v1alpha.PreparationService/GetCurrentPrepID',
            kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.GetCurrentPrepIDRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v1alpha_dot_preparation__service__pb2.GetCurrentPrepIDResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import datetime
import logging
import sys
from enum import Enum
from typing import List, Optional, Union

import grpc

import kaskada.formatters
import kaskada.kaskada.v1alpha.query_service_pb2 as query_pb
from kaskada.client import KASKADA_DEFAULT_SLICE, Client, get_client
from kaskada.slice_filters import SliceFilter
from kaskada.utils import get_timestamp, handleException, handleGrpcError

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger(__name__)


class ResponseType(Enum):
    FILE_TYPE_PARQUET = 1
    FILE_TYPE_CSV = 2


def queryV2(
    query: str,
    result_behavior: str = "all-results",
    response_as: ResponseType = ResponseType.FILE_TYPE_PARQUET,
    data_token_id: Optional[str] = None,
    dry_run: bool = False,
    changed_since_time: Union[str, datetime.datetime, None] = None,
    final_result_time: Union[str, datetime.datetime, None] = None,
    limits: Optional[query_pb.Query.Limits] = None,
    slice_filter: Optional[SliceFilter] = None,
    experimental: bool = False,
    client: Optional[Client] = None,
) -> query_pb.CreateQueryResponse:
    """
    Performs a query

    Args:
        query (str): The query to perform
        result_behavior (str, optional):
            Determines which results are returned. Either "all-results" (default), or "final-results" which returns
            only the final values for each entity.
        responsed_as (ResponseType):
            Determines how the response is returned.  Either "parquet" (default) or "csv".
        data_token_id (str, optional):
            Enables repeatable queries. Queries performed against the same dataToken are always ran the same input data.
        dry_run(bool, optional):
            When `True`, the query is validated and if there are no errors, the resultant schema is returned.
            No actual computation of results is performed.
        changed_since_time (datetime.datetime, optional):
            Configure the inclusive datetime after which results will be output.
        limits (pb.QueryRequest.Limits, optional):
            Configure limits on the output set.
        slice_filter (SliceFilter, optional):
            Enables slice filter. Currently, only slice entity percent filters are supported. Defaults to None.
        experimental (bool):
            When `True`, then experimental features are allowed. Data returned when using this flag is not
            guaranteed to be correct. Default to False
        client (Client, optional):
            The Kaskada Client. Defaults to kaskada.KASKADA_DEFAULT_CLIENT.

    Returns:
        compute_pb.QueryResponse: Response from the API
    """
    if slice_filter is None:
        slice_filter = KASKADA_DEFAULT_SLICE

    change_since_time = get_timestamp(changed_since_time)
    final_result_time = get_timestamp(final_result_time)

    try:
        client = get_client(client)
        query_options = {
            "dry_run": dry_run,
            "experimental_features": experimental,
            "stream_metrics": False,
            "presign_results": True,
        }

        query_request = {
            "expression": query,
            "data_token_id": data_token_id,
            "changed_since_time": change_since_time,
            "final_result_time": final_result_time,
            "limits": limits,
        }

        if result_behavior == "final-results":
            query_request["result_behavior"] = "RESULT_BEHAVIOR_FINAL_RESULTS"
            if final_result_time is not None:
                query_request[
                    "result_behavior"
                ] = "RESULT_BEHAVIOR_FINAL_RESULTS_AT_TIME"
        else:
            query_request["result_behavior"] = "RESULT_BEHAVIOR_ALL_RESULTS"

        query_request["as_files"] = {"file_type": response_as.name}

        if slice_filter is not None:
            query_request["slice"] = slice_filter.to_request()

        in_ipython = kaskada.formatters.in_ipython()
        if in_ipython:
            query_options["stream_metrics"] = True

        request_args = {"query": query_request, "query_options": query_options}
        request = query_pb.CreateQueryRequest(**request_args)
        logger.debug(f"Query Request: {request}")
        return execute_query(request, client)
    except grpc.RpcError as exec:
        handleGrpcError(exec)
    except Exception as exec:
        handleException(exec)


def execute_query(
    request: query_pb.CreateQueryRequest, client: Client
) -> query_pb.CreateQueryResponse:
    """Executes a query using the streaming request format"""
    response = query_pb.CreateQueryResponse()
    in_ipython = kaskada.formatters.in_ipython()
    if in_ipython:
        from IPython.display import clear_output, display
    responses = client.query_stub.CreateQuery(request, metadata=client.get_metadata())
    for resp in responses:
        response.MergeFrom(resp)
        logger.debug(f"Query Response: {response}")
        if in_ipython:
            clear_output(wait=True)
            display(response)

    if in_ipython:
        clear_output(wait=True)
    return response



from datetime import datetime

import pandas
from domonic.html import b, pre, td, tr
from google.protobuf.message import Message

import kaskada.kaskada.v1alpha.schema_pb2 as schema_pb2

"""
NOTES:

Use `hasattr(obj, prop) to test if an object has a property`
Use `obj.HasField(prop) to test if an object has a specific one-of`
"""

kaskada_fenl_primitive_types = {
    schema_pb2.DataType.PRIMITIVE_TYPE_BOOL: "bool",
    schema_pb2.DataType.PRIMITIVE_TYPE_DURATION_MICROSECOND: "duration_us",
    schema_pb2.DataType.PRIMITIVE_TYPE_DURATION_MILLISECOND: "duration_ms",
    schema_pb2.DataType.PRIMITIVE_TYPE_DURATION_NANOSECOND: "duration_ns",
    schema_pb2.DataType.PRIMITIVE_TYPE_DURATION_SECOND: "duration_s",
    schema_pb2.DataType.PRIMITIVE_TYPE_F16: "f16",
    schema_pb2.DataType.PRIMITIVE_TYPE_F32: "f32",
    schema_pb2.DataType.PRIMITIVE_TYPE_F64: "f64",
    schema_pb2.DataType.PRIMITIVE_TYPE_I16: "i16",
    schema_pb2.DataType.PRIMITIVE_TYPE_I32: "i32",
    schema_pb2.DataType.PRIMITIVE_TYPE_I64: "i64",
    schema_pb2.DataType.PRIMITIVE_TYPE_I8: "i8",
    schema_pb2.DataType.PRIMITIVE_TYPE_INTERVAL_DAY_TIME: "interval_days",
    schema_pb2.DataType.PRIMITIVE_TYPE_INTERVAL_YEAR_MONTH: "interval_months",
    schema_pb2.DataType.PRIMITIVE_TYPE_NULL: "null",
    schema_pb2.DataType.PRIMITIVE_TYPE_STRING: "string",
    schema_pb2.DataType.PRIMITIVE_TYPE_TIMESTAMP_MICROSECOND: "timestamp_us",
    schema_pb2.DataType.PRIMITIVE_TYPE_TIMESTAMP_MILLISECOND: "timestamp_ms",
    schema_pb2.DataType.PRIMITIVE_TYPE_TIMESTAMP_NANOSECOND: "timestamp_ns",
    schema_pb2.DataType.PRIMITIVE_TYPE_TIMESTAMP_SECOND: "timestamp_s",
    schema_pb2.DataType.PRIMITIVE_TYPE_U16: "u16",
    schema_pb2.DataType.PRIMITIVE_TYPE_U32: "u32",
    schema_pb2.DataType.PRIMITIVE_TYPE_U64: "u64",
    schema_pb2.DataType.PRIMITIVE_TYPE_U8: "u8",
}


def get_item(obj, keys, default=""):
    """
    Safely recursively gets an item from a dict, otherwise returns the default value

    Args:
        obj (dict of {str : Any}): Dictionary of (nested) key item pairs
        keys (list of str): The list of keys use to navigate to the desired item in the dictionary.
        default (str, optional): The default value to return if the item isn't found. Defaults to "".

    Returns:
        string: Either the found value or the default.
    """
    while len(keys) > 0:
        key = keys.pop(0)
        if key in obj:
            obj = obj[key]
        else:
            return default
    return obj


def appendChildIfNotNone(parent, child):
    if child is not None:
        parent.appendChild(child)


def appendChildren(parent, children):
    for child in children:
        appendChildIfNotNone(parent, child)


def html_obj_table_row(key, value):
    """
    Returns a html table row, where the key is bolded, and the value is output as code.
    """
    if value is None or value == "":
        return None
    return tr(td(b(key)), td(pre(value)))


def html_table_row(key, value):
    """
    Returns a html table row, where the key is bolded, and the value is output as is.
    """
    return tr(td(b(key)), td(value))


def html_obj_id_row(index, obj, props):
    obj_type = type(obj).__name__.lower()
    obj_name_key = f"{obj_type}_name"
    obj_name = get_item(props, [obj_name_key])
    return tr(td(pre(index)), td(pre(obj_name)))


def appendHtmlObjTableRowIfAttrExists(parent, obj, attr):
    if hasattr(obj, attr):
        elem = html_obj_table_row(attr, getattr(obj, attr))
        if elem is not None:
            parent.appendChild(elem)


def appendHtmlObjTableRowIfHasField(parent, obj, field):
    if obj.HasField(field):
        elem = html_obj_table_row(field, getattr(obj, field))
        if elem is not None:
            parent.appendChild(elem)


def convert_fenl_schema_field(field):
    """
    Converts a fenl field to a string representation.

    Args:
        field (schema_pb2.Schema_Field) : the input field

    Returns:
        str: The string representation of the fenl Field
    """
    return field.name, convert_fenl_datatype(field.data_type)


def convert_fenl_datatype(data_type):
    """
    Converts a fenl dataType to a string representation.

    Args:
        data_type (schema_pb2.DataType) : the input dataType

    Returns:
        str: The string representation of the fenl DataType
    """
    type_string = None
    if data_type.HasField("primitive"):
        type_string = kaskada_fenl_primitive_types.get(data_type.primitive, "<unknown>")
    elif data_type.HasField("struct"):
        type_string = "<struct>"
    elif data_type.HasField("window"):
        type_string = "<window>"
    return type_string


def get_schema_dataframe(obj):
    """
    Gets a dataframe from a schema object if it exists

    Args:
        obj (kaskada.fenl.v1alpha.schema_pb2.Schema | Any):
            Either a schema object or an object that has a schema or analysis object at the root level

    Returns:
        pandas.dataframe | None:
            Either a dataframe describing the schemea or None if no schema exists in the passed object.
    """
    if hasattr(obj, "schema"):
        obj = obj.schema

    if (not hasattr(obj, "fields")) or len(obj.fields) == 0:
        return None
    fields = {}
    for idx, field in enumerate(obj.fields):
        name, typeString = convert_fenl_schema_field(field)
        if len(name) > 0:
            fields[idx] = [name, typeString]
    return pandas.DataFrame.from_dict(
        data=fields, orient="index", columns=["column_name", "column_type"]
    )


def get_datetime(pb2_timestamp):
    """
    Converts a google.protobuf.timestamp_pb2 into a python datetime object.

    Args:
        pb2_timestamp (google.protobuf.timestamp_pb2): The protobuf timestamp.

    Returns:
        datetime: The python datetime.
    """
    return datetime.fromtimestamp(pb2_timestamp.seconds + pb2_timestamp.nanos / 1e9)


def get_classname(obj):
    return f"{obj.__module__}.{obj.__name__}"


def get_properties(obj):
    """
    Converts a protobuf response into a simple python dictionary.

    Args:
        obj (Any): A deserialized protobuf object.

    Returns:
        dict of {str : Any}: A nested set of keys and values with structure matching the protobuf input.
    """
    results = {}
    if isinstance(obj, Message):
        for field in obj.ListFields():
            name = field[0].name
            field_type = get_classname(type(field[1]))
            prop_value = field[1]
            if field_type in (int, float, bool):
                results[name] = prop_value
            elif field_type == "str":
                if prop_value != "":
                    results[name] = prop_value
            elif field_type == "google._upb._message.RepeatedCompositeContainer":
                if len(prop_value) > 0:
                    results[name] = [get_properties(sub_obj) for sub_obj in prop_value]
            elif field_type == "google.protobuf.timestamp_pb2.Timestamp":
                results[name] = get_datetime(prop_value).isoformat()
            elif field_type.startswith("pandas."):
                results[name] = prop_value
            elif isinstance(prop_value, Message):
                results[name] = get_properties(prop_value)
            else:
                results[name] = prop_value
    return results



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kaskada/kaskada/v1alpha/preparation_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from kaskada.kaskada.v1alpha import common_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1kaskada/kaskada/v1alpha/preparation_service.proto\x12\x17kaskada.kaskada.v1alpha\x1a$kaskada/kaskada/v1alpha/common.proto\"\xa4\x02\n\x12PrepareDataRequest\x12>\n\tfile_path\x18\x01 \x01(\x0b\x32!.kaskada.kaskada.v1alpha.FilePathR\x08\x66ilePath\x12<\n\x06\x63onfig\x18\x02 \x01(\x0b\x32$.kaskada.kaskada.v1alpha.TableConfigR\x06\x63onfig\x12,\n\x12output_path_prefix\x18\x03 \x01(\tR\x10outputPathPrefix\x12\x1f\n\x0b\x66ile_prefix\x18\x04 \x01(\tR\nfilePrefix\x12\x41\n\nslice_plan\x18\x05 \x01(\x0b\x32\".kaskada.kaskada.v1alpha.SlicePlanR\tslicePlan\"|\n\x13PrepareDataResponse\x12\x17\n\x07prep_id\x18\x01 \x01(\x05R\x06prepId\x12L\n\x0eprepared_files\x18\x02 \x03(\x0b\x32%.kaskada.kaskada.v1alpha.PreparedFileR\rpreparedFiles\"\x19\n\x17GetCurrentPrepIDRequest\"3\n\x18GetCurrentPrepIDResponse\x12\x17\n\x07prep_id\x18\x01 \x01(\x05R\x06prepId2\xf7\x01\n\x12PreparationService\x12h\n\x0bPrepareData\x12+.kaskada.kaskada.v1alpha.PrepareDataRequest\x1a,.kaskada.kaskada.v1alpha.PrepareDataResponse\x12w\n\x10GetCurrentPrepID\x12\x30.kaskada.kaskada.v1alpha.GetCurrentPrepIDRequest\x1a\x31.kaskada.kaskada.v1alpha.GetCurrentPrepIDResponseB\x87\x02\n\x1b\x63om.kaskada.kaskada.v1alphaB\x17PreparationServiceProtoP\x01ZQgithub.com/kaskada-ai/kaskada/gen/proto/go/kaskada/kaskada/v1alpha;kaskadav1alpha\xa2\x02\x03KKX\xaa\x02\x17Kaskada.Kaskada.V1alpha\xca\x02\x17Kaskada\\Kaskada\\V1alpha\xe2\x02#Kaskada\\Kaskada\\V1alpha\\GPBMetadata\xea\x02\x19Kaskada::Kaskada::V1alphab\x06proto3')



_PREPAREDATAREQUEST = DESCRIPTOR.message_types_by_name['PrepareDataRequest']
_PREPAREDATARESPONSE = DESCRIPTOR.message_types_by_name['PrepareDataResponse']
_GETCURRENTPREPIDREQUEST = DESCRIPTOR.message_types_by_name['GetCurrentPrepIDRequest']
_GETCURRENTPREPIDRESPONSE = DESCRIPTOR.message_types_by_name['GetCurrentPrepIDResponse']
PrepareDataRequest = _reflection.GeneratedProtocolMessageType('PrepareDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _PREPAREDATAREQUEST,
  '__module__' : 'kaskada.kaskada.v1alpha.preparation_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.PrepareDataRequest)
  })
_sym_db.RegisterMessage(PrepareDataRequest)

PrepareDataResponse = _reflection.GeneratedProtocolMessageType('PrepareDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _PREPAREDATARESPONSE,
  '__module__' : 'kaskada.kaskada.v1alpha.preparation_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.PrepareDataResponse)
  })
_sym_db.RegisterMessage(PrepareDataResponse)

GetCurrentPrepIDRequest = _reflection.GeneratedProtocolMessageType('GetCurrentPrepIDRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETCURRENTPREPIDREQUEST,
  '__module__' : 'kaskada.kaskada.v1alpha.preparation_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.GetCurrentPrepIDRequest)
  })
_sym_db.RegisterMessage(GetCurrentPrepIDRequest)

GetCurrentPrepIDResponse = _reflection.GeneratedProtocolMessageType('GetCurrentPrepIDResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETCURRENTPREPIDRESPONSE,
  '__module__' : 'kaskada.kaskada.v1alpha.preparation_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.GetCurrentPrepIDResponse)
  })
_sym_db.RegisterMessage(GetCurrentPrepIDResponse)

_PREPARATIONSERVICE = DESCRIPTOR.services_by_name['PreparationService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033com.kaskada.kaskada.v1alphaB\027PreparationServiceProtoP\001ZQgithub.com/kaskada-ai/kaskada/gen/proto/go/kaskada/kaskada/v1alpha;kaskadav1alpha\242\002\003KKX\252\002\027Kaskada.Kaskada.V1alpha\312\002\027Kaskada\\Kaskada\\V1alpha\342\002#Kaskada\\Kaskada\\V1alpha\\GPBMetadata\352\002\031Kaskada::Kaskada::V1alpha'
  _PREPAREDATAREQUEST._serialized_start=117
  _PREPAREDATAREQUEST._serialized_end=409
  _PREPAREDATARESPONSE._serialized_start=411
  _PREPAREDATARESPONSE._serialized_end=535
  _GETCURRENTPREPIDREQUEST._serialized_start=537
  _GETCURRENTPREPIDREQUEST._serialized_end=562
  _GETCURRENTPREPIDRESPONSE._serialized_start=564
  _GETCURRENTPREPIDRESPONSE._serialized_end=615
  _PREPARATIONSERVICE._serialized_start=618
  _PREPARATIONSERVICE._serialized_end=865
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kaskada/kaskada/v1alpha/file_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from kaskada.kaskada.v1alpha import common_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_common__pb2
from kaskada.kaskada.v1alpha import schema_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_schema__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*kaskada/kaskada/v1alpha/file_service.proto\x12\x17kaskada.kaskada.v1alpha\x1a$kaskada/kaskada/v1alpha/common.proto\x1a$kaskada/kaskada/v1alpha/schema.proto\"G\n\x0c\x46ileMetadata\x12\x37\n\x06schema\x18\x01 \x01(\x0b\x32\x1f.kaskada.kaskada.v1alpha.SchemaR\x06schema\"V\n\x12GetMetadataRequest\x12@\n\nfile_paths\x18\x01 \x03(\x0b\x32!.kaskada.kaskada.v1alpha.FilePathR\tfilePaths\"c\n\x13GetMetadataResponse\x12L\n\x0e\x66ile_metadatas\x18\x01 \x03(\x0b\x32%.kaskada.kaskada.v1alpha.FileMetadataR\rfileMetadatas\"\xb8\x01\n\x14MergeMetadataRequest\x12M\n\x0etable_metadata\x18\x01 \x01(\x0b\x32&.kaskada.kaskada.v1alpha.TableMetadataR\rtableMetadata\x12Q\n\x11new_file_metadata\x18\x02 \x03(\x0b\x32%.kaskada.kaskada.v1alpha.FileMetadataR\x0fnewFileMetadata\"\x90\x02\n\x15MergeMetadataResponse\x12Z\n\x15merged_table_metadata\x18\x01 \x01(\x0b\x32&.kaskada.kaskada.v1alpha.TableMetadataR\x13mergedTableMetadata\x12h\n\x10new_file_results\x18\x02 \x03(\x0b\x32>.kaskada.kaskada.v1alpha.MergeMetadataResponse.FileMergeResultR\x0enewFileResults\x1a\x31\n\x0f\x46ileMergeResult\x12\x1e\n\ncompatible\x18\x01 \x01(\x08R\ncompatible2\xe7\x01\n\x0b\x46ileService\x12h\n\x0bGetMetadata\x12+.kaskada.kaskada.v1alpha.GetMetadataRequest\x1a,.kaskada.kaskada.v1alpha.GetMetadataResponse\x12n\n\rMergeMetadata\x12-.kaskada.kaskada.v1alpha.MergeMetadataRequest\x1a..kaskada.kaskada.v1alpha.MergeMetadataResponseB\x80\x02\n\x1b\x63om.kaskada.kaskada.v1alphaB\x10\x46ileServiceProtoP\x01ZQgithub.com/kaskada-ai/kaskada/gen/proto/go/kaskada/kaskada/v1alpha;kaskadav1alpha\xa2\x02\x03KKX\xaa\x02\x17Kaskada.Kaskada.V1alpha\xca\x02\x17Kaskada\\Kaskada\\V1alpha\xe2\x02#Kaskada\\Kaskada\\V1alpha\\GPBMetadata\xea\x02\x19Kaskada::Kaskada::V1alphab\x06proto3')



_FILEMETADATA = DESCRIPTOR.message_types_by_name['FileMetadata']
_GETMETADATAREQUEST = DESCRIPTOR.message_types_by_name['GetMetadataRequest']
_GETMETADATARESPONSE = DESCRIPTOR.message_types_by_name['GetMetadataResponse']
_MERGEMETADATAREQUEST = DESCRIPTOR.message_types_by_name['MergeMetadataRequest']
_MERGEMETADATARESPONSE = DESCRIPTOR.message_types_by_name['MergeMetadataResponse']
_MERGEMETADATARESPONSE_FILEMERGERESULT = _MERGEMETADATARESPONSE.nested_types_by_name['FileMergeResult']
FileMetadata = _reflection.GeneratedProtocolMessageType('FileMetadata', (_message.Message,), {
  'DESCRIPTOR' : _FILEMETADATA,
  '__module__' : 'kaskada.kaskada.v1alpha.file_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.FileMetadata)
  })
_sym_db.RegisterMessage(FileMetadata)

GetMetadataRequest = _reflection.GeneratedProtocolMessageType('GetMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETMETADATAREQUEST,
  '__module__' : 'kaskada.kaskada.v1alpha.file_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.GetMetadataRequest)
  })
_sym_db.RegisterMessage(GetMetadataRequest)

GetMetadataResponse = _reflection.GeneratedProtocolMessageType('GetMetadataResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETMETADATARESPONSE,
  '__module__' : 'kaskada.kaskada.v1alpha.file_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.GetMetadataResponse)
  })
_sym_db.RegisterMessage(GetMetadataResponse)

MergeMetadataRequest = _reflection.GeneratedProtocolMessageType('MergeMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _MERGEMETADATAREQUEST,
  '__module__' : 'kaskada.kaskada.v1alpha.file_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.MergeMetadataRequest)
  })
_sym_db.RegisterMessage(MergeMetadataRequest)

MergeMetadataResponse = _reflection.GeneratedProtocolMessageType('MergeMetadataResponse', (_message.Message,), {

  'FileMergeResult' : _reflection.GeneratedProtocolMessageType('FileMergeResult', (_message.Message,), {
    'DESCRIPTOR' : _MERGEMETADATARESPONSE_FILEMERGERESULT,
    '__module__' : 'kaskada.kaskada.v1alpha.file_service_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.MergeMetadataResponse.FileMergeResult)
    })
  ,
  'DESCRIPTOR' : _MERGEMETADATARESPONSE,
  '__module__' : 'kaskada.kaskada.v1alpha.file_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.MergeMetadataResponse)
  })
_sym_db.RegisterMessage(MergeMetadataResponse)
_sym_db.RegisterMessage(MergeMetadataResponse.FileMergeResult)

_FILESERVICE = DESCRIPTOR.services_by_name['FileService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033com.kaskada.kaskada.v1alphaB\020FileServiceProtoP\001ZQgithub.com/kaskada-ai/kaskada/gen/proto/go/kaskada/kaskada/v1alpha;kaskadav1alpha\242\002\003KKX\252\002\027Kaskada.Kaskada.V1alpha\312\002\027Kaskada\\Kaskada\\V1alpha\342\002#Kaskada\\Kaskada\\V1alpha\\GPBMetadata\352\002\031Kaskada::Kaskada::V1alpha'
  _FILEMETADATA._serialized_start=147
  _FILEMETADATA._serialized_end=218
  _GETMETADATAREQUEST._serialized_start=220
  _GETMETADATAREQUEST._serialized_end=306
  _GETMETADATARESPONSE._serialized_start=308
  _GETMETADATARESPONSE._serialized_end=407
  _MERGEMETADATAREQUEST._serialized_start=410
  _MERGEMETADATAREQUEST._serialized_end=594
  _MERGEMETADATARESPONSE._serialized_start=597
  _MERGEMETADATARESPONSE._serialized_end=869
  _MERGEMETADATARESPONSE_FILEMERGERESULT._serialized_start=820
  _MERGEMETADATARESPONSE_FILEMERGERESULT._serialized_end=869
  _FILESERVICE._serialized_start=872
  _FILESERVICE._serialized_end=1103
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kaskada/kaskada/v1alpha/table_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from kaskada.kaskada.v1alpha import common_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_common__pb2
from kaskada.kaskada.v1alpha import schema_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_schema__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+kaskada/kaskada/v1alpha/table_service.proto\x12\x17kaskada.kaskada.v1alpha\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a$kaskada/kaskada/v1alpha/common.proto\x1a$kaskada/kaskada/v1alpha/schema.proto\x1a\x17validate/validate.proto\"\x80\x06\n\x05Table\x12\x1e\n\x08table_id\x18\x01 \x01(\tB\x03\xe0\x41\x03R\x07tableId\x12&\n\ntable_name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\ttableName\x12\x31\n\x10time_column_name\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x0etimeColumnName\x12<\n\x16\x65ntity_key_column_name\x18\x04 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x13\x65ntityKeyColumnName\x12U\n\x13subsort_column_name\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.StringValueB\x07\xfa\x42\x04r\x02\x10\x01R\x11subsortColumnName\x12\x1f\n\x0bgrouping_id\x18\x06 \x01(\tR\ngroupingId\x12@\n\x0b\x63reate_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x03\xe0\x41\x03R\ncreateTime\x12@\n\x0bupdate_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x03\xe0\x41\x03R\nupdateTime\x12\x1d\n\x07version\x18\t \x01(\x03\x42\x03\xe0\x41\x03R\x07version\x12<\n\x06schema\x18\n \x01(\x0b\x32\x1f.kaskada.kaskada.v1alpha.SchemaB\x03\xe0\x41\x03R\x06schema\x12\"\n\nfile_count\x18\x0b \x01(\x03\x42\x03\xe0\x41\x03R\tfileCount\x12M\n\x0ctable_source\x18\x0c \x01(\x0b\x32*.kaskada.kaskada.v1alpha.Table.TableSourceR\x0btableSource\x1a\x61\n\x0bTableSource\x12H\n\x07kaskada\x18\x01 \x01(\x0b\x32,.kaskada.kaskada.v1alpha.Table.KaskadaSourceH\x00R\x07kaskadaB\x08\n\x06source\x1a\x0f\n\rKaskadaSource\"s\n\x11ListTablesRequest\x12\x16\n\x06search\x18\x01 \x01(\tR\x06search\x12\'\n\tpage_size\x18\x02 \x01(\x05\x42\n\xfa\x42\x07\x1a\x05\x18\xe8\x07(\x00R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"\xc6\x01\n\x12ListTablesResponse\x12\x36\n\x06tables\x18\x01 \x03(\x0b\x32\x1e.kaskada.kaskada.v1alpha.TableR\x06tables\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\x12P\n\x0frequest_details\x18\x03 \x01(\x0b\x32\'.kaskada.kaskada.v1alpha.RequestDetailsR\x0erequestDetails\"9\n\x0fGetTableRequest\x12&\n\ntable_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\ttableName\"\x9a\x01\n\x10GetTableResponse\x12\x34\n\x05table\x18\x01 \x01(\x0b\x32\x1e.kaskada.kaskada.v1alpha.TableR\x05table\x12P\n\x0frequest_details\x18\x02 \x01(\x0b\x32\'.kaskada.kaskada.v1alpha.RequestDetailsR\x0erequestDetails\"T\n\x12\x43reateTableRequest\x12>\n\x05table\x18\x01 \x01(\x0b\x32\x1e.kaskada.kaskada.v1alpha.TableB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x05table\"\x9d\x01\n\x13\x43reateTableResponse\x12\x34\n\x05table\x18\x01 \x01(\x0b\x32\x1e.kaskada.kaskada.v1alpha.TableR\x05table\x12P\n\x0frequest_details\x18\x02 \x01(\x0b\x32\'.kaskada.kaskada.v1alpha.RequestDetailsR\x0erequestDetails\"R\n\x12\x44\x65leteTableRequest\x12&\n\ntable_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\ttableName\x12\x14\n\x05\x66orce\x18\x02 \x01(\x08R\x05\x66orce\"\x8b\x01\n\x13\x44\x65leteTableResponse\x12\"\n\rdata_token_id\x18\x01 \x01(\tR\x0b\x64\x61taTokenId\x12P\n\x0frequest_details\x18\x02 \x01(\x0b\x32\'.kaskada.kaskada.v1alpha.RequestDetailsR\x0erequestDetails\"\x92\x01\n\x0fLoadDataRequest\x12&\n\ntable_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\ttableName\x12\x43\n\nfile_input\x18\x02 \x01(\x0b\x32\".kaskada.kaskada.v1alpha.FileInputH\x00R\tfileInputB\x12\n\x0bsource_data\x12\x03\xf8\x42\x01\"\x88\x01\n\x10LoadDataResponse\x12\"\n\rdata_token_id\x18\x01 \x01(\tR\x0b\x64\x61taTokenId\x12P\n\x0frequest_details\x18\x02 \x01(\x0b\x32\'.kaskada.kaskada.v1alpha.RequestDetailsR\x0erequestDetails2\xbe\x05\n\x0cTableService\x12~\n\nListTables\x12*.kaskada.kaskada.v1alpha.ListTablesRequest\x1a+.kaskada.kaskada.v1alpha.ListTablesResponse\"\x17\x82\xd3\xe4\x93\x02\x11\x12\x0f/v1alpha/tables\x12\x87\x01\n\x08GetTable\x12(.kaskada.kaskada.v1alpha.GetTableRequest\x1a).kaskada.kaskada.v1alpha.GetTableResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/v1alpha/tables/{table_name=*}\x12\x88\x01\n\x0b\x43reateTable\x12+.kaskada.kaskada.v1alpha.CreateTableRequest\x1a,.kaskada.kaskada.v1alpha.CreateTableResponse\"\x1e\x82\xd3\xe4\x93\x02\x18:\x05table\"\x0f/v1alpha/tables\x12\x90\x01\n\x0b\x44\x65leteTable\x12+.kaskada.kaskada.v1alpha.DeleteTableRequest\x1a,.kaskada.kaskada.v1alpha.DeleteTableResponse\"&\x82\xd3\xe4\x93\x02 *\x1e/v1alpha/tables/{table_name=*}\x12\x85\x01\n\x08LoadData\x12(.kaskada.kaskada.v1alpha.LoadDataRequest\x1a).kaskada.kaskada.v1alpha.LoadDataResponse\"$\x82\xd3\xe4\x93\x02\x1e:\x01*\"\x19/v1alpha/tables/load_dataB\x81\x02\n\x1b\x63om.kaskada.kaskada.v1alphaB\x11TableServiceProtoP\x01ZQgithub.com/kaskada-ai/kaskada/gen/proto/go/kaskada/kaskada/v1alpha;kaskadav1alpha\xa2\x02\x03KKX\xaa\x02\x17Kaskada.Kaskada.V1alpha\xca\x02\x17Kaskada\\Kaskada\\V1alpha\xe2\x02#Kaskada\\Kaskada\\V1alpha\\GPBMetadata\xea\x02\x19Kaskada::Kaskada::V1alphab\x06proto3')



_TABLE = DESCRIPTOR.message_types_by_name['Table']
_TABLE_TABLESOURCE = _TABLE.nested_types_by_name['TableSource']
_TABLE_KASKADASOURCE = _TABLE.nested_types_by_name['KaskadaSource']
_LISTTABLESREQUEST = DESCRIPTOR.message_types_by_name['ListTablesRequest']
_LISTTABLESRESPONSE = DESCRIPTOR.message_types_by_name['ListTablesResponse']
_GETTABLEREQUEST = DESCRIPTOR.message_types_by_name['GetTableRequest']
_GETTABLERESPONSE = DESCRIPTOR.message_types_by_name['GetTableResponse']
_CREATETABLEREQUEST = DESCRIPTOR.message_types_by_name['CreateTableRequest']
_CREATETABLERESPONSE = DESCRIPTOR.message_types_by_name['CreateTableResponse']
_DELETETABLEREQUEST = DESCRIPTOR.message_types_by_name['DeleteTableRequest']
_DELETETABLERESPONSE = DESCRIPTOR.message_types_by_name['DeleteTableResponse']
_LOADDATAREQUEST = DESCRIPTOR.message_types_by_name['LoadDataRequest']
_LOADDATARESPONSE = DESCRIPTOR.message_types_by_name['LoadDataResponse']
Table = _reflection.GeneratedProtocolMessageType('Table', (_message.Message,), {

  'TableSource' : _reflection.GeneratedProtocolMessageType('TableSource', (_message.Message,), {
    'DESCRIPTOR' : _TABLE_TABLESOURCE,
    '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Table.TableSource)
    })
  ,

  'KaskadaSource' : _reflection.GeneratedProtocolMessageType('KaskadaSource', (_message.Message,), {
    'DESCRIPTOR' : _TABLE_KASKADASOURCE,
    '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Table.KaskadaSource)
    })
  ,
  'DESCRIPTOR' : _TABLE,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Table)
  })
_sym_db.RegisterMessage(Table)
_sym_db.RegisterMessage(Table.TableSource)
_sym_db.RegisterMessage(Table.KaskadaSource)

ListTablesRequest = _reflection.GeneratedProtocolMessageType('ListTablesRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTTABLESREQUEST,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.ListTablesRequest)
  })
_sym_db.RegisterMessage(ListTablesRequest)

ListTablesResponse = _reflection.GeneratedProtocolMessageType('ListTablesResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTTABLESRESPONSE,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.ListTablesResponse)
  })
_sym_db.RegisterMessage(ListTablesResponse)

GetTableRequest = _reflection.GeneratedProtocolMessageType('GetTableRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTABLEREQUEST,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.GetTableRequest)
  })
_sym_db.RegisterMessage(GetTableRequest)

GetTableResponse = _reflection.GeneratedProtocolMessageType('GetTableResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTABLERESPONSE,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.GetTableResponse)
  })
_sym_db.RegisterMessage(GetTableResponse)

CreateTableRequest = _reflection.GeneratedProtocolMessageType('CreateTableRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATETABLEREQUEST,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.CreateTableRequest)
  })
_sym_db.RegisterMessage(CreateTableRequest)

CreateTableResponse = _reflection.GeneratedProtocolMessageType('CreateTableResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATETABLERESPONSE,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.CreateTableResponse)
  })
_sym_db.RegisterMessage(CreateTableResponse)

DeleteTableRequest = _reflection.GeneratedProtocolMessageType('DeleteTableRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETETABLEREQUEST,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.DeleteTableRequest)
  })
_sym_db.RegisterMessage(DeleteTableRequest)

DeleteTableResponse = _reflection.GeneratedProtocolMessageType('DeleteTableResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETETABLERESPONSE,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.DeleteTableResponse)
  })
_sym_db.RegisterMessage(DeleteTableResponse)

LoadDataRequest = _reflection.GeneratedProtocolMessageType('LoadDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOADDATAREQUEST,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.LoadDataRequest)
  })
_sym_db.RegisterMessage(LoadDataRequest)

LoadDataResponse = _reflection.GeneratedProtocolMessageType('LoadDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _LOADDATARESPONSE,
  '__module__' : 'kaskada.kaskada.v1alpha.table_service_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.LoadDataResponse)
  })
_sym_db.RegisterMessage(LoadDataResponse)

_TABLESERVICE = DESCRIPTOR.services_by_name['TableService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033com.kaskada.kaskada.v1alphaB\021TableServiceProtoP\001ZQgithub.com/kaskada-ai/kaskada/gen/proto/go/kaskada/kaskada/v1alpha;kaskadav1alpha\242\002\003KKX\252\002\027Kaskada.Kaskada.V1alpha\312\002\027Kaskada\\Kaskada\\V1alpha\342\002#Kaskada\\Kaskada\\V1alpha\\GPBMetadata\352\002\031Kaskada::Kaskada::V1alpha'
  _TABLE.fields_by_name['table_id']._options = None
  _TABLE.fields_by_name['table_id']._serialized_options = b'\340A\003'
  _TABLE.fields_by_name['table_name']._options = None
  _TABLE.fields_by_name['table_name']._serialized_options = b'\372B\004r\002\020\001'
  _TABLE.fields_by_name['time_column_name']._options = None
  _TABLE.fields_by_name['time_column_name']._serialized_options = b'\372B\004r\002\020\001'
  _TABLE.fields_by_name['entity_key_column_name']._options = None
  _TABLE.fields_by_name['entity_key_column_name']._serialized_options = b'\372B\004r\002\020\001'
  _TABLE.fields_by_name['subsort_column_name']._options = None
  _TABLE.fields_by_name['subsort_column_name']._serialized_options = b'\372B\004r\002\020\001'
  _TABLE.fields_by_name['create_time']._options = None
  _TABLE.fields_by_name['create_time']._serialized_options = b'\340A\003'
  _TABLE.fields_by_name['update_time']._options = None
  _TABLE.fields_by_name['update_time']._serialized_options = b'\340A\003'
  _TABLE.fields_by_name['version']._options = None
  _TABLE.fields_by_name['version']._serialized_options = b'\340A\003'
  _TABLE.fields_by_name['schema']._options = None
  _TABLE.fields_by_name['schema']._serialized_options = b'\340A\003'
  _TABLE.fields_by_name['file_count']._options = None
  _TABLE.fields_by_name['file_count']._serialized_options = b'\340A\003'
  _LISTTABLESREQUEST.fields_by_name['page_size']._options = None
  _LISTTABLESREQUEST.fields_by_name['page_size']._serialized_options = b'\372B\007\032\005\030\350\007(\000'
  _GETTABLEREQUEST.fields_by_name['table_name']._options = None
  _GETTABLEREQUEST.fields_by_name['table_name']._serialized_options = b'\372B\004r\002\020\001'
  _CREATETABLEREQUEST.fields_by_name['table']._options = None
  _CREATETABLEREQUEST.fields_by_name['table']._serialized_options = b'\372B\005\212\001\002\020\001'
  _DELETETABLEREQUEST.fields_by_name['table_name']._options = None
  _DELETETABLEREQUEST.fields_by_name['table_name']._serialized_options = b'\372B\004r\002\020\001'
  _LOADDATAREQUEST.oneofs_by_name['source_data']._options = None
  _LOADDATAREQUEST.oneofs_by_name['source_data']._serialized_options = b'\370B\001'
  _LOADDATAREQUEST.fields_by_name['table_name']._options = None
  _LOADDATAREQUEST.fields_by_name['table_name']._serialized_options = b'\372B\004r\002\020\001'
  _TABLESERVICE.methods_by_name['ListTables']._options = None
  _TABLESERVICE.methods_by_name['ListTables']._serialized_options = b'\202\323\344\223\002\021\022\017/v1alpha/tables'
  _TABLESERVICE.methods_by_name['GetTable']._options = None
  _TABLESERVICE.methods_by_name['GetTable']._serialized_options = b'\202\323\344\223\002 \022\036/v1alpha/tables/{table_name=*}'
  _TABLESERVICE.methods_by_name['CreateTable']._options = None
  _TABLESERVICE.methods_by_name['CreateTable']._serialized_options = b'\202\323\344\223\002\030:\005table\"\017/v1alpha/tables'
  _TABLESERVICE.methods_by_name['DeleteTable']._options = None
  _TABLESERVICE.methods_by_name['DeleteTable']._serialized_options = b'\202\323\344\223\002 *\036/v1alpha/tables/{table_name=*}'
  _TABLESERVICE.methods_by_name['LoadData']._options = None
  _TABLESERVICE.methods_by_name['LoadData']._serialized_options = b'\202\323\344\223\002\036:\001*\"\031/v1alpha/tables/load_data'
  _TABLE._serialized_start=302
  _TABLE._serialized_end=1070
  _TABLE_TABLESOURCE._serialized_start=956
  _TABLE_TABLESOURCE._serialized_end=1053
  _TABLE_KASKADASOURCE._serialized_start=1055
  _TABLE_KASKADASOURCE._serialized_end=1070
  _LISTTABLESREQUEST._serialized_start=1072
  _LISTTABLESREQUEST._serialized_end=1187
  _LISTTABLESRESPONSE._serialized_start=1190
  _LISTTABLESRESPONSE._serialized_end=1388
  _GETTABLEREQUEST._serialized_start=1390
  _GETTABLEREQUEST._serialized_end=1447
  _GETTABLERESPONSE._serialized_start=1450
  _GETTABLERESPONSE._serialized_end=1604
  _CREATETABLEREQUEST._serialized_start=1606
  _CREATETABLEREQUEST._serialized_end=1690
  _CREATETABLERESPONSE._serialized_start=1693
  _CREATETABLERESPONSE._serialized_end=1850
  _DELETETABLEREQUEST._serialized_start=1852
  _DELETETABLEREQUEST._serialized_end=1934
  _DELETETABLERESPONSE._serialized_start=1937
  _DELETETABLERESPONSE._serialized_end=2076
  _LOADDATAREQUEST._serialized_start=2079
  _LOADDATAREQUEST._serialized_end=2225
  _LOADDATARESPONSE._serialized_start=2228
  _LOADDATARESPONSE._serialized_end=2364
  _TABLESERVICE._serialized_start=2367
  _TABLESERVICE._serialized_end=3069
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kaskada/kaskada/v1alpha/plan.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from kaskada.kaskada.v1alpha import common_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_common__pb2
from kaskada.kaskada.v1alpha import schema_pb2 as kaskada_dot_kaskada_dot_v1alpha_dot_schema__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"kaskada/kaskada/v1alpha/plan.proto\x12\x17kaskada.kaskada.v1alpha\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a$kaskada/kaskada/v1alpha/common.proto\x1a$kaskada/kaskada/v1alpha/schema.proto\"\xba\x02\n\x0b\x43omputePlan\x12Z\n\x13per_entity_behavior\x18\x01 \x01(\x0e\x32*.kaskada.kaskada.v1alpha.PerEntityBehaviorR\x11perEntityBehavior\x12\x46\n\noperations\x18\x02 \x03(\x0b\x32&.kaskada.kaskada.v1alpha.OperationPlanR\noperations\x12)\n\x10primary_grouping\x18\x03 \x01(\tR\x0fprimaryGrouping\x12\\\n\x19primary_grouping_key_type\x18\x04 \x01(\x0b\x32!.kaskada.kaskada.v1alpha.DataTypeR\x16primaryGroupingKeyType\"\xc6\x12\n\rOperationPlan\x12I\n\x0b\x65xpressions\x18\x01 \x03(\x0b\x32\'.kaskada.kaskada.v1alpha.ExpressionPlanR\x0b\x65xpressions\x12J\n\x04scan\x18\x02 \x01(\x0b\x32\x34.kaskada.kaskada.v1alpha.OperationPlan.ScanOperationH\x00R\x04scan\x12M\n\x05merge\x18\x03 \x01(\x0b\x32\x35.kaskada.kaskada.v1alpha.OperationPlan.MergeOperationH\x00R\x05merge\x12P\n\x06select\x18\x04 \x01(\x0b\x32\x36.kaskada.kaskada.v1alpha.OperationPlan.SelectOperationH\x00R\x06select\x12J\n\x04tick\x18\x05 \x01(\x0b\x32\x34.kaskada.kaskada.v1alpha.OperationPlan.TickOperationH\x00R\x04tick\x12T\n\x08with_key\x18\x06 \x01(\x0b\x32\x37.kaskada.kaskada.v1alpha.OperationPlan.WithKeyOperationH\x00R\x07withKey\x12\x66\n\x0elookup_request\x18\x07 \x01(\x0b\x32=.kaskada.kaskada.v1alpha.OperationPlan.LookupRequestOperationH\x00R\rlookupRequest\x12i\n\x0flookup_response\x18\x08 \x01(\x0b\x32>.kaskada.kaskada.v1alpha.OperationPlan.LookupResponseOperationH\x00R\x0elookupResponse\x12T\n\x08shift_to\x18\t \x01(\x0b\x32\x37.kaskada.kaskada.v1alpha.OperationPlan.ShiftToOperationH\x00R\x07shiftTo\x12]\n\x0bshift_until\x18\n \x01(\x0b\x32:.kaskada.kaskada.v1alpha.OperationPlan.ShiftUntilOperationH\x00R\nshiftUntil\x1a\xc5\x01\n\rScanOperation\x12\x38\n\x08table_id\x18\x01 \x01(\x0b\x32\x1d.kaskada.kaskada.v1alpha.UuidR\x07tableId\x12\x37\n\x06schema\x18\x02 \x01(\x0b\x32\x1f.kaskada.kaskada.v1alpha.SchemaR\x06schema\x12\x41\n\nslice_plan\x18\x03 \x01(\x0b\x32\".kaskada.kaskada.v1alpha.SlicePlanR\tslicePlan\x1a:\n\x0eMergeOperation\x12\x12\n\x04left\x18\x01 \x01(\rR\x04left\x12\x14\n\x05right\x18\x02 \x01(\rR\x05right\x1aq\n\x0fSelectOperation\x12\x14\n\x05input\x18\x01 \x01(\rR\x05input\x12H\n\tcondition\x18\x02 \x01(\x0b\x32*.kaskada.kaskada.v1alpha.OperationInputRefR\tcondition\x1a\x89\x01\n\x10WithKeyOperation\x12\x14\n\x05input\x18\x01 \x01(\rR\x05input\x12\x43\n\x07new_key\x18\x02 \x01(\x0b\x32*.kaskada.kaskada.v1alpha.OperationInputRefR\x06newKey\x12\x1a\n\x08grouping\x18\x03 \x01(\tR\x08grouping\x1a\xd4\x02\n\rTickOperation\x12]\n\x08\x62\x65havior\x18\x01 \x01(\x0e\x32\x41.kaskada.kaskada.v1alpha.OperationPlan.TickOperation.TickBehaviorR\x08\x62\x65havior\x12\x14\n\x05input\x18\x02 \x01(\rR\x05input\"\xcd\x01\n\x0cTickBehavior\x12\x1d\n\x19TICK_BEHAVIOR_UNSPECIFIED\x10\x00\x12\x1a\n\x16TICK_BEHAVIOR_FINISHED\x10\x01\x12\x18\n\x14TICK_BEHAVIOR_HOURLY\x10\x02\x12\x17\n\x13TICK_BEHAVIOR_DAILY\x10\x03\x12\x19\n\x15TICK_BEHAVIOR_MONTHLY\x10\x04\x12\x18\n\x14TICK_BEHAVIOR_YEARLY\x10\x05\x12\x1a\n\x16TICK_BEHAVIOR_MINUTELY\x10\x06\x1a\x9b\x01\n\x16LookupRequestOperation\x12+\n\x11primary_operation\x18\x01 \x01(\rR\x10primaryOperation\x12T\n\x10\x66oreign_key_hash\x18\x02 \x01(\x0b\x32*.kaskada.kaskada.v1alpha.OperationInputRefR\x0e\x66oreignKeyHash\x1a\xa2\x01\n\x17LookupResponseOperation\x12+\n\x11\x66oreign_operation\x18\x01 \x01(\rR\x10\x66oreignOperation\x12Z\n\x13requesting_key_hash\x18\x02 \x01(\x0b\x32*.kaskada.kaskada.v1alpha.OperationInputRefR\x11requestingKeyHash\x1a\xb2\x01\n\x10ShiftToOperation\x12\x14\n\x05input\x18\x01 \x01(\rR\x05input\x12H\n\x08\x63omputed\x18\x02 \x01(\x0b\x32*.kaskada.kaskada.v1alpha.OperationInputRefH\x00R\x08\x63omputed\x12\x36\n\x07literal\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\x07literalB\x06\n\x04time\x1au\n\x13ShiftUntilOperation\x12\x14\n\x05input\x18\x01 \x01(\rR\x05input\x12H\n\tcondition\x18\x02 \x01(\x0b\x32*.kaskada.kaskada.v1alpha.OperationInputRefR\tconditionB\n\n\x08operator\"\x86\x03\n\x0e\x45xpressionPlan\x12\x1c\n\targuments\x18\x01 \x03(\rR\targuments\x12\x42\n\x0bresult_type\x18\x02 \x01(\x0b\x32!.kaskada.kaskada.v1alpha.DataTypeR\nresultType\x12\x16\n\x06output\x18\x03 \x01(\x08R\x06output\x12\"\n\x0binstruction\x18\x04 \x01(\tH\x00R\x0binstruction\x12\x42\n\x05input\x18\x05 \x01(\x0b\x32*.kaskada.kaskada.v1alpha.OperationInputRefH\x00R\x05input\x12<\n\x07literal\x18\x06 \x01(\x0b\x32 .kaskada.kaskada.v1alpha.LiteralH\x00R\x07literal\x12H\n\nlate_bound\x18\x07 \x01(\x0e\x32\'.kaskada.kaskada.v1alpha.LateBoundValueH\x00R\tlateBoundB\n\n\x08operator\"\xb5\n\n\x07Literal\x12\x14\n\x04\x62ool\x18\x08 \x01(\x08H\x00R\x04\x62ool\x12\x14\n\x04int8\x18\t \x01(\x05H\x00R\x04int8\x12\x16\n\x05int16\x18\n \x01(\x05H\x00R\x05int16\x12\x16\n\x05int32\x18\x0b \x01(\x05H\x00R\x05int32\x12\x16\n\x05int64\x18\x0c \x01(\x03H\x00R\x05int64\x12\x16\n\x05uint8\x18\r \x01(\rH\x00R\x05uint8\x12\x18\n\x06uint16\x18\x0e \x01(\rH\x00R\x06uint16\x12\x18\n\x06uint32\x18\x0f \x01(\rH\x00R\x06uint32\x12\x18\n\x06uint64\x18\x10 \x01(\x04H\x00R\x06uint64\x12\x1a\n\x07\x66loat32\x18\x11 \x01(\x02H\x00R\x07\x66loat32\x12\x1a\n\x07\x66loat64\x18\x12 \x01(\x01H\x00R\x07\x66loat64\x12O\n\ttimestamp\x18\x13 \x01(\x0b\x32/.kaskada.kaskada.v1alpha.Literal.TimestampValueH\x00R\ttimestamp\x12\x18\n\x06\x64\x61te32\x18\x14 \x01(\x05H\x00R\x06\x64\x61te32\x12\x18\n\x06\x64\x61te64\x18\x15 \x01(\x03H\x00R\x06\x64\x61te64\x12\x46\n\x06time32\x18\x16 \x01(\x0b\x32,.kaskada.kaskada.v1alpha.Literal.Time32ValueH\x00R\x06time32\x12\x46\n\x06time64\x18\x17 \x01(\x0b\x32,.kaskada.kaskada.v1alpha.Literal.Time64ValueH\x00R\x06time64\x12L\n\x08\x64uration\x18\x18 \x01(\x0b\x32..kaskada.kaskada.v1alpha.Literal.DurationValueH\x00R\x08\x64uration\x12\x63\n\x11interval_day_time\x18\x19 \x01(\x0b\x32\x35.kaskada.kaskada.v1alpha.Literal.IntervalDayTimeValueH\x00R\x0fintervalDayTime\x12)\n\x0finterval_months\x18\x1a \x01(\x05H\x00R\x0eintervalMonths\x12\x14\n\x04utf8\x18\x1b \x01(\tH\x00R\x04utf8\x12\x46\n\x06record\x18\x1c \x01(\x0b\x32,.kaskada.kaskada.v1alpha.Literal.RecordValueH\x00R\x06record\x1a\x85\x01\n\x0eTimestampValue\x12\x31\n\x05value\x18\x01 \x01(\x0b\x32\x1b.google.protobuf.Int64ValueR\x05value\x12\x12\n\x04unit\x18\x02 \x01(\tR\x04unit\x12,\n\x02tz\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValueR\x02tz\x1a\x37\n\x0bTime32Value\x12\x14\n\x05value\x18\x01 \x01(\x05R\x05value\x12\x12\n\x04unit\x18\x02 \x01(\tR\x04unit\x1a\x37\n\x0bTime64Value\x12\x14\n\x05value\x18\x01 \x01(\x03R\x05value\x12\x12\n\x04unit\x18\x02 \x01(\tR\x04unit\x1a\x39\n\rDurationValue\x12\x14\n\x05value\x18\x01 \x01(\x03R\x05value\x12\x12\n\x04unit\x18\x02 \x01(\tR\x04unit\x1a>\n\x14IntervalDayTimeValue\x12\x14\n\x05start\x18\x01 \x01(\x05R\x05start\x12\x10\n\x03\x65nd\x18\x02 \x01(\x05R\x03\x65nd\x1aG\n\x0bRecordValue\x12\x38\n\x06values\x18\x01 \x03(\x0b\x32 .kaskada.kaskada.v1alpha.LiteralR\x06valuesB\t\n\x07literal\"\x94\x05\n\x11OperationInputRef\x12/\n\x13producing_operation\x18\x01 \x01(\rR\x12producingOperation\x12U\n\nkey_column\x18\x02 \x01(\x0e\x32\x34.kaskada.kaskada.v1alpha.OperationInputRef.KeyColumnH\x00R\tkeyColumn\x12\x31\n\x13producer_expression\x18\x03 \x01(\rH\x00R\x12producerExpression\x12\x39\n\x0bscan_record\x18\x04 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00R\nscanRecord\x12,\n\x04tick\x18\x07 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00R\x04tick\x12!\n\x0cinput_column\x18\x05 \x01(\rR\x0binputColumn\x12^\n\rinterpolation\x18\x06 \x01(\x0e\x32\x38.kaskada.kaskada.v1alpha.OperationInputRef.InterpolationR\rinterpolation\"m\n\tKeyColumn\x12\x1a\n\x16KEY_COLUMN_UNSPECIFIED\x10\x00\x12\x13\n\x0fKEY_COLUMN_TIME\x10\x01\x12\x16\n\x12KEY_COLUMN_SUBSORT\x10\x02\x12\x17\n\x13KEY_COLUMN_KEY_HASH\x10\x03\"_\n\rInterpolation\x12\x1d\n\x19INTERPOLATION_UNSPECIFIED\x10\x00\x12\x16\n\x12INTERPOLATION_NULL\x10\x01\x12\x17\n\x13INTERPOLATION_AS_OF\x10\x02\x42\x08\n\x06\x63olumn*\x7f\n\x0eLateBoundValue\x12 \n\x1cLATE_BOUND_VALUE_UNSPECIFIED\x10\x00\x12\'\n#LATE_BOUND_VALUE_CHANGED_SINCE_TIME\x10\x01\x12\"\n\x1eLATE_BOUND_VALUE_FINAL_AT_TIME\x10\x02\x42\xf9\x01\n\x1b\x63om.kaskada.kaskada.v1alphaB\tPlanProtoP\x01ZQgithub.com/kaskada-ai/kaskada/gen/proto/go/kaskada/kaskada/v1alpha;kaskadav1alpha\xa2\x02\x03KKX\xaa\x02\x17Kaskada.Kaskada.V1alpha\xca\x02\x17Kaskada\\Kaskada\\V1alpha\xe2\x02#Kaskada\\Kaskada\\V1alpha\\GPBMetadata\xea\x02\x19Kaskada::Kaskada::V1alphab\x06proto3')

_LATEBOUNDVALUE = DESCRIPTOR.enum_types_by_name['LateBoundValue']
LateBoundValue = enum_type_wrapper.EnumTypeWrapper(_LATEBOUNDVALUE)
LATE_BOUND_VALUE_UNSPECIFIED = 0
LATE_BOUND_VALUE_CHANGED_SINCE_TIME = 1
LATE_BOUND_VALUE_FINAL_AT_TIME = 2


_COMPUTEPLAN = DESCRIPTOR.message_types_by_name['ComputePlan']
_OPERATIONPLAN = DESCRIPTOR.message_types_by_name['OperationPlan']
_OPERATIONPLAN_SCANOPERATION = _OPERATIONPLAN.nested_types_by_name['ScanOperation']
_OPERATIONPLAN_MERGEOPERATION = _OPERATIONPLAN.nested_types_by_name['MergeOperation']
_OPERATIONPLAN_SELECTOPERATION = _OPERATIONPLAN.nested_types_by_name['SelectOperation']
_OPERATIONPLAN_WITHKEYOPERATION = _OPERATIONPLAN.nested_types_by_name['WithKeyOperation']
_OPERATIONPLAN_TICKOPERATION = _OPERATIONPLAN.nested_types_by_name['TickOperation']
_OPERATIONPLAN_LOOKUPREQUESTOPERATION = _OPERATIONPLAN.nested_types_by_name['LookupRequestOperation']
_OPERATIONPLAN_LOOKUPRESPONSEOPERATION = _OPERATIONPLAN.nested_types_by_name['LookupResponseOperation']
_OPERATIONPLAN_SHIFTTOOPERATION = _OPERATIONPLAN.nested_types_by_name['ShiftToOperation']
_OPERATIONPLAN_SHIFTUNTILOPERATION = _OPERATIONPLAN.nested_types_by_name['ShiftUntilOperation']
_EXPRESSIONPLAN = DESCRIPTOR.message_types_by_name['ExpressionPlan']
_LITERAL = DESCRIPTOR.message_types_by_name['Literal']
_LITERAL_TIMESTAMPVALUE = _LITERAL.nested_types_by_name['TimestampValue']
_LITERAL_TIME32VALUE = _LITERAL.nested_types_by_name['Time32Value']
_LITERAL_TIME64VALUE = _LITERAL.nested_types_by_name['Time64Value']
_LITERAL_DURATIONVALUE = _LITERAL.nested_types_by_name['DurationValue']
_LITERAL_INTERVALDAYTIMEVALUE = _LITERAL.nested_types_by_name['IntervalDayTimeValue']
_LITERAL_RECORDVALUE = _LITERAL.nested_types_by_name['RecordValue']
_OPERATIONINPUTREF = DESCRIPTOR.message_types_by_name['OperationInputRef']
_OPERATIONPLAN_TICKOPERATION_TICKBEHAVIOR = _OPERATIONPLAN_TICKOPERATION.enum_types_by_name['TickBehavior']
_OPERATIONINPUTREF_KEYCOLUMN = _OPERATIONINPUTREF.enum_types_by_name['KeyColumn']
_OPERATIONINPUTREF_INTERPOLATION = _OPERATIONINPUTREF.enum_types_by_name['Interpolation']
ComputePlan = _reflection.GeneratedProtocolMessageType('ComputePlan', (_message.Message,), {
  'DESCRIPTOR' : _COMPUTEPLAN,
  '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.ComputePlan)
  })
_sym_db.RegisterMessage(ComputePlan)

OperationPlan = _reflection.GeneratedProtocolMessageType('OperationPlan', (_message.Message,), {

  'ScanOperation' : _reflection.GeneratedProtocolMessageType('ScanOperation', (_message.Message,), {
    'DESCRIPTOR' : _OPERATIONPLAN_SCANOPERATION,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan.ScanOperation)
    })
  ,

  'MergeOperation' : _reflection.GeneratedProtocolMessageType('MergeOperation', (_message.Message,), {
    'DESCRIPTOR' : _OPERATIONPLAN_MERGEOPERATION,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan.MergeOperation)
    })
  ,

  'SelectOperation' : _reflection.GeneratedProtocolMessageType('SelectOperation', (_message.Message,), {
    'DESCRIPTOR' : _OPERATIONPLAN_SELECTOPERATION,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan.SelectOperation)
    })
  ,

  'WithKeyOperation' : _reflection.GeneratedProtocolMessageType('WithKeyOperation', (_message.Message,), {
    'DESCRIPTOR' : _OPERATIONPLAN_WITHKEYOPERATION,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan.WithKeyOperation)
    })
  ,

  'TickOperation' : _reflection.GeneratedProtocolMessageType('TickOperation', (_message.Message,), {
    'DESCRIPTOR' : _OPERATIONPLAN_TICKOPERATION,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan.TickOperation)
    })
  ,

  'LookupRequestOperation' : _reflection.GeneratedProtocolMessageType('LookupRequestOperation', (_message.Message,), {
    'DESCRIPTOR' : _OPERATIONPLAN_LOOKUPREQUESTOPERATION,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan.LookupRequestOperation)
    })
  ,

  'LookupResponseOperation' : _reflection.GeneratedProtocolMessageType('LookupResponseOperation', (_message.Message,), {
    'DESCRIPTOR' : _OPERATIONPLAN_LOOKUPRESPONSEOPERATION,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan.LookupResponseOperation)
    })
  ,

  'ShiftToOperation' : _reflection.GeneratedProtocolMessageType('ShiftToOperation', (_message.Message,), {
    'DESCRIPTOR' : _OPERATIONPLAN_SHIFTTOOPERATION,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan.ShiftToOperation)
    })
  ,

  'ShiftUntilOperation' : _reflection.GeneratedProtocolMessageType('ShiftUntilOperation', (_message.Message,), {
    'DESCRIPTOR' : _OPERATIONPLAN_SHIFTUNTILOPERATION,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan.ShiftUntilOperation)
    })
  ,
  'DESCRIPTOR' : _OPERATIONPLAN,
  '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationPlan)
  })
_sym_db.RegisterMessage(OperationPlan)
_sym_db.RegisterMessage(OperationPlan.ScanOperation)
_sym_db.RegisterMessage(OperationPlan.MergeOperation)
_sym_db.RegisterMessage(OperationPlan.SelectOperation)
_sym_db.RegisterMessage(OperationPlan.WithKeyOperation)
_sym_db.RegisterMessage(OperationPlan.TickOperation)
_sym_db.RegisterMessage(OperationPlan.LookupRequestOperation)
_sym_db.RegisterMessage(OperationPlan.LookupResponseOperation)
_sym_db.RegisterMessage(OperationPlan.ShiftToOperation)
_sym_db.RegisterMessage(OperationPlan.ShiftUntilOperation)

ExpressionPlan = _reflection.GeneratedProtocolMessageType('ExpressionPlan', (_message.Message,), {
  'DESCRIPTOR' : _EXPRESSIONPLAN,
  '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.ExpressionPlan)
  })
_sym_db.RegisterMessage(ExpressionPlan)

Literal = _reflection.GeneratedProtocolMessageType('Literal', (_message.Message,), {

  'TimestampValue' : _reflection.GeneratedProtocolMessageType('TimestampValue', (_message.Message,), {
    'DESCRIPTOR' : _LITERAL_TIMESTAMPVALUE,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Literal.TimestampValue)
    })
  ,

  'Time32Value' : _reflection.GeneratedProtocolMessageType('Time32Value', (_message.Message,), {
    'DESCRIPTOR' : _LITERAL_TIME32VALUE,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Literal.Time32Value)
    })
  ,

  'Time64Value' : _reflection.GeneratedProtocolMessageType('Time64Value', (_message.Message,), {
    'DESCRIPTOR' : _LITERAL_TIME64VALUE,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Literal.Time64Value)
    })
  ,

  'DurationValue' : _reflection.GeneratedProtocolMessageType('DurationValue', (_message.Message,), {
    'DESCRIPTOR' : _LITERAL_DURATIONVALUE,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Literal.DurationValue)
    })
  ,

  'IntervalDayTimeValue' : _reflection.GeneratedProtocolMessageType('IntervalDayTimeValue', (_message.Message,), {
    'DESCRIPTOR' : _LITERAL_INTERVALDAYTIMEVALUE,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Literal.IntervalDayTimeValue)
    })
  ,

  'RecordValue' : _reflection.GeneratedProtocolMessageType('RecordValue', (_message.Message,), {
    'DESCRIPTOR' : _LITERAL_RECORDVALUE,
    '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
    # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Literal.RecordValue)
    })
  ,
  'DESCRIPTOR' : _LITERAL,
  '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.Literal)
  })
_sym_db.RegisterMessage(Literal)
_sym_db.RegisterMessage(Literal.TimestampValue)
_sym_db.RegisterMessage(Literal.Time32Value)
_sym_db.RegisterMessage(Literal.Time64Value)
_sym_db.RegisterMessage(Literal.DurationValue)
_sym_db.RegisterMessage(Literal.IntervalDayTimeValue)
_sym_db.RegisterMessage(Literal.RecordValue)

OperationInputRef = _reflection.GeneratedProtocolMessageType('OperationInputRef', (_message.Message,), {
  'DESCRIPTOR' : _OPERATIONINPUTREF,
  '__module__' : 'kaskada.kaskada.v1alpha.plan_pb2'
  # @@protoc_insertion_point(class_scope:kaskada.kaskada.v1alpha.OperationInputRef)
  })
_sym_db.RegisterMessage(OperationInputRef)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033com.kaskada.kaskada.v1alphaB\tPlanProtoP\001ZQgithub.com/kaskada-ai/kaskada/gen/proto/go/kaskada/kaskada/v1alpha;kaskadav1alpha\242\002\003KKX\252\002\027Kaskada.Kaskada.V1alpha\312\002\027Kaskada\\Kaskada\\V1alpha\342\002#Kaskada\\Kaskada\\V1alpha\\GPBMetadata\352\002\031Kaskada::Kaskada::V1alpha'
  _LATEBOUNDVALUE._serialized_start=5319
  _LATEBOUNDVALUE._serialized_end=5446
  _COMPUTEPLAN._serialized_start=234
  _COMPUTEPLAN._serialized_end=548
  _OPERATIONPLAN._serialized_start=551
  _OPERATIONPLAN._serialized_end=2925
  _OPERATIONPLAN_SCANOPERATION._serialized_start=1435
  _OPERATIONPLAN_SCANOPERATION._serialized_end=1632
  _OPERATIONPLAN_MERGEOPERATION._serialized_start=1634
  _OPERATIONPLAN_MERGEOPERATION._serialized_end=1692
  _OPERATIONPLAN_SELECTOPERATION._serialized_start=1694
  _OPERATIONPLAN_SELECTOPERATION._serialized_end=1807
  _OPERATIONPLAN_WITHKEYOPERATION._serialized_start=1810
  _OPERATIONPLAN_WITHKEYOPERATION._serialized_end=1947
  _OPERATIONPLAN_TICKOPERATION._serialized_start=1950
  _OPERATIONPLAN_TICKOPERATION._serialized_end=2290
  _OPERATIONPLAN_TICKOPERATION_TICKBEHAVIOR._serialized_start=2085
  _OPERATIONPLAN_TICKOPERATION_TICKBEHAVIOR._serialized_end=2290
  _OPERATIONPLAN_LOOKUPREQUESTOPERATION._serialized_start=2293
  _OPERATIONPLAN_LOOKUPREQUESTOPERATION._serialized_end=2448
  _OPERATIONPLAN_LOOKUPRESPONSEOPERATION._serialized_start=2451
  _OPERATIONPLAN_LOOKUPRESPONSEOPERATION._serialized_end=2613
  _OPERATIONPLAN_SHIFTTOOPERATION._serialized_start=2616
  _OPERATIONPLAN_SHIFTTOOPERATION._serialized_end=2794
  _OPERATIONPLAN_SHIFTUNTILOPERATION._serialized_start=2796
  _OPERATIONPLAN_SHIFTUNTILOPERATION._serialized_end=2913
  _EXPRESSIONPLAN._serialized_start=2928
  _EXPRESSIONPLAN._serialized_end=3318
  _LITERAL._serialized_start=3321
  _LITERAL._serialized_end=4654
  _LITERAL_TIMESTAMPVALUE._serialized_start=4200
  _LITERAL_TIMESTAMPVALUE._serialized_end=4333
  _LITERAL_TIME32VALUE._serialized_start=4335
  _LITERAL_TIME32VALUE._serialized_end=4390
  _LITERAL_TIME64VALUE._serialized_start=4392
  _LITERAL_TIME64VALUE._serialized_end=4447
  _LITERAL_DURATIONVALUE._serialized_start=4449
  _LITERAL_DURATIONVALUE._serialized_end=4506
  _LITERAL_INTERVALDAYTIMEVALUE._serialized_start=4508
  _LITERAL_INTERVALDAYTIMEVALUE._serialized_end=4570
  _LITERAL_RECORDVALUE._serialized_start=4572
  _LITERAL_RECORDVALUE._serialized_end=4643
  _OPERATIONINPUTREF._serialized_start=4657
  _OPERATIONINPUTREF._serialized_end=5317
  _OPERATIONINPUTREF_KEYCOLUMN._serialized_start=5101
  _OPERATIONINPUTREF_KEYCOLUMN._serialized_end=5210
  _OPERATIONINPUTREF_INTERPOLATION._serialized_start=5212
  _OPERATIONINPUTREF_INTERPOLATION._serialized_end=5307
# @@protoc_insertion_point(module_scope)



import datetime
import re
import unicodedata
from typing import Union

import grpc
from google.protobuf.timestamp_pb2 import Timestamp
from google.rpc import error_details_pb2
from grpc_status import rpc_status

import kaskada.kaskada.v1alpha.fenl_diagnostics_pb2 as kaskada_error_details_pb2


def normalize_col_name(column: str) -> str:
    """
    Normalize column name by replacing invalid characters with underscore
    strips accents and make lowercase
    """
    n = re.sub(r"[ ,;{}()\n\t=]+", "_", column.lower())
    return unicodedata.normalize("NFKD", n).encode("ASCII", "ignore").decode()


def get_timestamp(time: Union[str, datetime.datetime, None]):
    timestamp = None
    if isinstance(time, str):
        timestamp = Timestamp()
        timestamp.FromJsonString(time)
    elif isinstance(time, datetime.datetime):
        timestamp = Timestamp()
        timestamp.FromDatetime(time)
    elif time is not None:
        raise Exception(
            "Invalid type for timestamp. Expected `str` or `datetime.datetime`."
        )
    return timestamp


def handleGrpcError(rpc_error: grpc.Call):
    """
    All methods calling `handleGrpcError` do so after detecting a RpcError.  Currently
    all our grpc calls perform a `UnaryUnaryMultiCallable` method, docs linked here:
    https://grpc.github.io/grpc/python/_modules/grpc.html#UnaryUnaryMultiCallable

    Calling a `UnaryUnaryMultiCallable` method results in one of the following:
        Returns:
            The response value for the RPC.

        Raises:
            RpcError: Indicating that the RPC terminated with non-OK status. The
            raised RpcError will also be a grpc.Call for the RPC affording the RPC's
            metadata, status code, and details.

    Methods on a `grpc.Call` object include the following.  Full docs here:
    https://grpc.github.io/grpc/python/_modules/grpc.html#Call

        code(): The StatusCode value for the RPC.
        details(): The details string of the RPC.

    The `grpc_status.rpc_status.from_call(call)` method helps extract details
    objects from `grpc.Call` objects. Docs here:
    https://grpc.github.io/grpc/python/_modules/grpc_status/rpc_status.html#from_call

        Args:
            call: A grpc.Call instance.

        Returns:
            A google.rpc.status.Status message representing the status of the RPC.

        Raises:
            ValueError: If the gRPC call's code or details are inconsistent with the
            status code and message inside of the google.rpc.status.Status.
    """
    errorMessage = "An error occurred in your request.\n\tError Code: {}\n".format(
        rpc_error.code().name
    )
    try:
        status = rpc_status.from_call(rpc_error)
        if status:
            unpacked = []
            errorMessage += "\tError Message: {}\n".format(status.message)
            for detail in status.details:
                try:
                    val = unpack_details(detail)
                    unpacked.append(val)
                except Exception:
                    # maybe report back to wren in the future
                    pass
            if len(unpacked) > 0:
                errorMessage += "Details: \n"
                for val in unpacked:
                    errorMessage += "\t{}\n".format(val)
            raise Exception(errorMessage) from None
    except ValueError:
        # maybe report back to wren in the future
        pass
    errorMessage += "\tError Message: {}\n".format(rpc_error.details())
    raise Exception(errorMessage) from None


def unpack_details(grpc_detail):
    """Unpack a grpc status detail field (which is a 'google.protobuf.Any' type).
    `Unpack()` checks the descriptor of the passed-in message object against the stored one
    and returns False if they don't match and does not attempt any unpacking; True otherwise.
        Source:
            https://github.com/protocolbuffers/protobuf/blob/master/python/google/protobuf/internal/well_known_types.py#L81
            https://github.com/protocolbuffers/protobuf/blob/master/python/google/protobuf/internal/python_message.py#L1135

        Raises:
            google.protobuf.message.DecodeError: If it can't deserialize the message object

    `Is()` checks if the `Any` message represents the given protocol buffer type.
    """
    if grpc_detail.Is(error_details_pb2.BadRequest.DESCRIPTOR):
        val = error_details_pb2.BadRequest()
        grpc_detail.Unpack(val)
        return val
    elif grpc_detail.Is(error_details_pb2.PreconditionFailure.DESCRIPTOR):
        val = error_details_pb2.PreconditionFailure()
        grpc_detail.Unpack(val)
        return val
    elif grpc_detail.Is(error_details_pb2.RetryInfo.DESCRIPTOR):
        val = error_details_pb2.RetryInfo()
        grpc_detail.Unpack(val)
        return val
    elif grpc_detail.Is(error_details_pb2.DebugInfo.DESCRIPTOR):
        val = error_details_pb2.DebugInfo()
        grpc_detail.Unpack(val)
        return val
    elif grpc_detail.Is(error_details_pb2.QuotaFailure.DESCRIPTOR):
        val = error_details_pb2.QuotaFailure()
        grpc_detail.Unpack(val)
        return val
    elif grpc_detail.Is(error_details_pb2.RequestInfo.DESCRIPTOR):
        val = error_details_pb2.RequestInfo()
        grpc_detail.Unpack(val)
        return val
    elif grpc_detail.Is(error_details_pb2.ResourceInfo.DESCRIPTOR):
        val = error_details_pb2.ResourceInfo()
        grpc_detail.Unpack(val)
        return val
    elif grpc_detail.Is(error_details_pb2.Help.DESCRIPTOR):
        val = error_details_pb2.Help()
        grpc_detail.Unpack(val)
        return val
    elif grpc_detail.Is(error_details_pb2.LocalizedMessage.DESCRIPTOR):
        val = error_details_pb2.LocalizedMessage()
        grpc_detail.Unpack(val)
        return val
    elif grpc_detail.Is(kaskada_error_details_pb2.FenlDiagnostics.DESCRIPTOR):
        val = kaskada_error_details_pb2.FenlDiagnostics()
        grpc_detail.Unpack(val)
        error_msgs = []
        for fenl_diagnostic in val.fenl_diagnostics:
            error_msgs.append(fenl_diagnostic.formatted.replace("\n", "\n\t"))
        return "\n".join(error_msgs)
    else:
        raise ValueError(grpc_detail.type_url)


def handleException(e: Exception):
    raise Exception("An exception occurred: {}".format(e)) from None



import logging
import sys
from pathlib import Path
from typing import Any, Dict, Optional, Union

import google.protobuf.wrappers_pb2 as wrappers
import grpc

import kaskada.kaskada.v1alpha.common_pb2 as common_pb
import kaskada.kaskada.v1alpha.table_service_pb2 as table_pb
from kaskada.client import Client, get_client
from kaskada.utils import handleException, handleGrpcError

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger(__name__)


def get_table_name(
    table: Union[
        table_pb.Table, table_pb.CreateTableResponse, table_pb.GetTableResponse, str
    ]
) -> str:
    """
    Gets the table name from either the table protobuf, create table response, get table response, or a string

    Args:
        table (Union[table_pb.Table, table_pb.CreateTableResponse, table_pb.GetTableResponse, str]):
            The target table object

    Returns:
        str: The table name (None if unable to match)
    """
    table_name = None
    if isinstance(table, table_pb.Table):
        table_name = table.table_name
    elif isinstance(table, table_pb.CreateTableResponse) or isinstance(
        table, table_pb.GetTableResponse
    ):
        table_name = table.table.table_name
    elif isinstance(table, str):
        table_name = table

    if table_name is None:
        raise Exception(
            "invalid table parameter provided. the table parameter must be the table object, table response from the \
                SDK, or the table name"
        )

    return table_name


def list_tables(
    search: Optional[str] = None, client: Optional[Client] = None
) -> table_pb.ListTablesResponse:
    """
    Lists all tables the user has access to

    Args:
        search (str, optional): The search parameter to filter list by. Defaults to None.
        client (Client, optional): The Kaskada Client. Defaults to kaskada.KASKADA_DEFAULT_CLIENT.

    Returns:
        table_pb.ListTablesResponse: Response from the API
    """
    try:
        client = get_client(client)
        req = table_pb.ListTablesRequest(
            search=search,
        )
        logger.debug(f"List Tables Request: {req}")
        return client.table_stub.ListTables(req, metadata=client.get_metadata())
    except grpc.RpcError as e:
        handleGrpcError(e)
    except Exception as e:
        handleException(e)


def get_table(
    table: Union[
        table_pb.Table, table_pb.CreateTableResponse, table_pb.GetTableResponse, str
    ],
    client: Optional[Client] = None,
) -> table_pb.GetTableResponse:
    """
    Gets a table by name

    Args:
        table (Union[Table, CreateTableResponse, GetTableResponse, str]): The target table object
        client (Client, optional): The Kaskada Client. Defaults to kaskada.KASKADA_DEFAULT_CLIENT.

    Returns:
        table_pb.GetTableResponse: Response from the API
    """
    try:
        table_name = get_table_name(table)
        client = get_client(client)
        req = table_pb.GetTableRequest(table_name=table_name)
        logger.debug(f"Get Tables Request: {req}")
        return client.table_stub.GetTable(req, metadata=client.get_metadata())
    except grpc.RpcError as e:
        handleGrpcError(e)
    except Exception as e:
        handleException(e)


def create_table(
    table_name: str,
    time_column_name: str,
    entity_key_column_name: str,
    subsort_column_name: Optional[str] = None,
    grouping_id: Optional[str] = None,
    client: Optional[Client] = None,
) -> table_pb.CreateTableResponse:
    """
    Creates a table

    Args:
        table_name (str):
            The name of the table
        time_column_name (str):
            The time column
        entity_key_column_name (str):
            The entity key column
        subsort_column_name (str, optional):
            The subsort column. Defaults to None and Kaskada will generate a subsort column for the data.
        grouping_id (str, optional):
            The grouping id. Defaults to None.
        client (Client, optional):
            The Kaskada Client. Defaults to kaskada.KASKADA_DEFAULT_CLIENT.

    Returns:
        table_pb.CreateTableResponse: Response from the API
    """
    try:
        client = get_client(client)
        table_args: Dict[str, Any] = {
            "table_name": table_name,
            "time_column_name": time_column_name,
            "entity_key_column_name": entity_key_column_name,
        }
        if grouping_id is not None:
            table_args["grouping_id"] = grouping_id
        if subsort_column_name is not None:
            table_args["subsort_column_name"] = wrappers.StringValue(
                value=subsort_column_name
            )
        req = table_pb.CreateTableRequest(table=table_pb.Table(**table_args))
        logger.debug(f"Create Tables Request: {req}")
        return client.table_stub.CreateTable(req, metadata=client.get_metadata())
    except grpc.RpcError as e:
        handleGrpcError(e)
    except Exception as e:
        handleException(e)


def delete_table(
    table: Union[
        table_pb.Table, table_pb.CreateTableResponse, table_pb.GetTableResponse, str
    ],
    client: Optional[Client] = None,
    force: bool = False,
) -> table_pb.DeleteTableResponse:
    """
    Deletes a table referenced by name

    Args:
        table (Union[Table, CreateTableResponse, GetTableResponse, str]): The target table object
        client (Client, optional): The Kaskada Client. Defaults to kaskada.KASKADA_DEFAULT_CLIENT.

    Returns:
        table_pb.DeleteTableResponse: Response from the API
    """

    try:
        table_name = get_table_name(table)
        client = get_client(client)
        req = table_pb.DeleteTableRequest(table_name=table_name, force=force)
        logger.debug(f"Delete Tables Request: {req}")
        return client.table_stub.DeleteTable(req, metadata=client.get_metadata())
    except grpc.RpcError as e:
        handleGrpcError(e)
    except Exception as e:
        handleException(e)


def load(
    table_name: str,
    file: str,
    client: Optional[Client] = None,
) -> table_pb.LoadDataResponse:
    """Loads a local file to a table. The type of file is inferred from the extension.

    Args:
        table_name (str): The name of the target table
        file (str): The path to a file (absolute or relative)
        client (Optional[Client], optional): The Kaskada Client. Defaults to None.

    Returns:
        table_pb.LoadDataResponse: Response from API
    """
    try:
        client = get_client(client)
        path = Path(file).absolute()
        extension = path.suffix.lower()
        file_type = ""
        if extension == ".parquet":
            file_type = "FILE_TYPE_PARQUET"
        elif extension == ".csv":
            file_type = "FILE_TYPE_CSV"
        else:
            raise ValueError(
                "invalid file type provided. only .parquet or .csv accepted"
            )

        input = common_pb.FileInput(
            file_type=file_type,
            uri=f"file://{path}",
        )
        req = table_pb.LoadDataRequest(table_name=table_name, file_input=input)
        logger.debug(f"Load Tables Request: {req}")
        return client.table_stub.LoadData(req, metadata=client.get_metadata())
    except grpc.RpcError as e:
        handleGrpcError(e)
    except Exception as e:
        handleException(e)



import logging
import sys
from typing import Optional

import grpc
from google.protobuf.message import Message

import kaskada.compute
import kaskada.formatters
import kaskada.kaskada.v1alpha.query_service_pb2 as query_pb
from kaskada.client import Client, get_client
from kaskada.utils import handleException, handleGrpcError

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger(__name__)


class QueryResource(object):
    CURRENT_DATA_TOKEN = ""

    def __init__(self, query: query_pb.Query):
        self.query = query

    def to_query_request(self):
        result = {
            "query_id": self.query.query_id,
            "expression": self.query.expression,
            "data_token_id": self.query.data_token_id,
            "changed_since_time": self.query.changed_since_time,
            "final_result_time": self.query.final_result_time,
            "limits": self.query.limits,
            "result_behavior": self.query.result_behavior,
        }
        if isinstance(self.query, Message):
            if self.query.HasField("as_files"):
                result["as_files"] = self.query.as_files

        return result

    def run(
        self,
        data_token=None,
        dry_run=False,
        experimental_features=False,
        client: Optional[Client] = None,
    ):
        try:
            client = get_client(client)
            query_options = {
                "dry_run": dry_run,
                "experimental_features": experimental_features,
                "stream_metrics": False,
                "presign_results": True,
            }
            query_request = self.to_query_request()
            in_ipython = kaskada.formatters.in_ipython()
            if in_ipython:
                query_options["stream_metrics"] = True
            if data_token is not None:
                query_request["data_token_id"] = {"value": data_token}
            request_args = {
                "query": query_request,
                "query_options": query_options,
            }
            request = query_pb.CreateQueryRequest(**request_args)
            logger.debug(f"Create Query Request: {request}")
            return kaskada.compute.execute_query(request, client)
        except grpc.RpcError as e:
            handleGrpcError(e)
        except Exception as e:
            handleException(e)

    def run_with_latest(self, dry_run=False, experimental_features=False):
        return self.run(
            data_token=QueryResource.CURRENT_DATA_TOKEN,
            dry_run=dry_run,
            experimental_features=experimental_features,
        )


def get_query(query_id: str, client: Optional[Client] = None):
    """
    Gets a query by query ID
    Args:
        query_id (str): The target query ID
        client (Client, optional): The Kaskada Client. Defaults to kaskada.KASKADA_DEFAULT_CLIENT.

    Raises:
        NotImplementedError
    """
    try:
        client = get_client(client)
        req = query_pb.GetQueryRequest(
            query_id=query_id,
        )
        logger.debug(f"Get Query Request: {req}")
        resp = client.query_stub.GetQuery(req, metadata=client.get_metadata())
        return QueryResource(resp.query)
    except grpc.RpcError as e:
        handleGrpcError(e)
    except Exception as e:
        handleException(e)


def list_queries(search: Optional[str] = None, client: Optional[Client] = None):
    """
    Lists all queries the user has previously performed

    Args:
        search (str): The search parameter to filter queries by. Defaults to None.
        client (Client, optional): The Kaskada Client. Defaults to kaskada.KASKADA_DEFAULT_CLIENT.

    Raises:
        NotImplementedError
    """
    try:
        client = get_client(client)
        req = query_pb.ListQueriesRequest(
            search=search,
        )
        logger.debug(f"List Query Request: {req}")
        return client.query_stub.ListQueries(req, metadata=client.get_metadata())

    except grpc.RpcError as e:
        handleGrpcError(e)
    except Exception as e:
        handleException(e)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from kaskada.kaskada.v2alpha import query_service_pb2 as kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2


class QueryServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateQuery = channel.unary_unary(
                '/kaskada.kaskada.v2alpha.QueryService/CreateQuery',
                request_serializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.CreateQueryRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.CreateQueryResponse.FromString,
                )
        self.DeleteQuery = channel.unary_unary(
                '/kaskada.kaskada.v2alpha.QueryService/DeleteQuery',
                request_serializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.DeleteQueryRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.DeleteQueryResponse.FromString,
                )
        self.GetQuery = channel.unary_unary(
                '/kaskada.kaskada.v2alpha.QueryService/GetQuery',
                request_serializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.GetQueryRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.GetQueryResponse.FromString,
                )
        self.ListQueries = channel.unary_unary(
                '/kaskada.kaskada.v2alpha.QueryService/ListQueries',
                request_serializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.ListQueriesRequest.SerializeToString,
                response_deserializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.ListQueriesResponse.FromString,
                )


class QueryServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateQuery(self, request, context):
        """Creates a Query for processing
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteQuery(self, request, context):
        """Deletes a query, stopping any processing on it.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetQuery(self, request, context):
        """Gets a Query.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListQueries(self, request, context):
        """Lists queries.

        The response includes all Queries defined for the Kaskada account.
        If a search string is provided, only Queries matching the search string
        are returned.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QueryServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateQuery,
                    request_deserializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.CreateQueryRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.CreateQueryResponse.SerializeToString,
            ),
            'DeleteQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteQuery,
                    request_deserializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.DeleteQueryRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.DeleteQueryResponse.SerializeToString,
            ),
            'GetQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.GetQuery,
                    request_deserializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.GetQueryRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.GetQueryResponse.SerializeToString,
            ),
            'ListQueries': grpc.unary_unary_rpc_method_handler(
                    servicer.ListQueries,
                    request_deserializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.ListQueriesRequest.FromString,
                    response_serializer=kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.ListQueriesResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'kaskada.kaskada.v2alpha.QueryService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class QueryService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v2alpha.QueryService/CreateQuery',
            kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.CreateQueryRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.CreateQueryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v2alpha.QueryService/DeleteQuery',
            kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.DeleteQueryRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.DeleteQueryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v2alpha.QueryService/GetQuery',
            kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.GetQueryRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.GetQueryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListQueries(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/kaskada.kaskada.v2alpha.QueryService/ListQueries',
            kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.ListQueriesRequest.SerializeToString,
            kaskada_dot_kaskada_dot_v2alpha_dot_query__service__pb2.ListQueriesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import kaskada.compute


def arg_to_response_type(arg: str) -> kaskada.compute.ResponseType:
    arg = arg.lower()
    if arg == "csv":
        return kaskada.compute.ResponseType.FILE_TYPE_CSV
    elif arg == "parquet":
        return kaskada.compute.ResponseType.FILE_TYPE_PARQUET
    else:
        raise ValueError(
            'invalid response type provided. only "csv" or "parquet" supported'
        )



from __future__ import print_function

import os

import IPython
import pandas
from IPython.core.error import UsageError
from IPython.core.magic import Magics, cell_magic, line_cell_magic, magics_class
from IPython.core.magic_arguments import argument, magic_arguments, parse_argstring

import kaskada.client as client
import kaskada.compute as compute
import kaskada.kaskada.v1alpha.query_service_pb2 as query_pb
from fenlmagic.utils import arg_to_response_type


class QueryResult(object):
    dataframe: pandas.DataFrame = None

    def __init__(
        self, query: str, query_response: query_pb.CreateQueryResponse  # type: ignore[valid-type]
    ):
        self.query = query
        self.query_response = query_response

    def set_dataframe(self, dataframe: pandas.DataFrame):
        self.dataframe = dataframe


@magics_class
class FenlMagics(Magics):

    client = None

    def __init__(self, shell, client):
        super(FenlMagics, self).__init__(shell)
        self.client = client

    @magic_arguments()
    @argument(
        "--changed-since-time",
        help="Time bound (inclusive) after which results will be produced.",
    )
    @argument(
        "--data-token",
        help="A data token to run queries against. Enables repeatable queries.",
    )
    @argument("--debug", default=False, help="Shows debugging information")
    @argument(
        "--dry-run",
        default=False,
        help="When `True`, the query is validated and if there are no errors, the resultant schema is returned. \
            No actual computation of results is performed.",
    )
    @argument(
        "--experimental",
        help="When `True`, then experimental features are allowed. \
            Data returned when using this flag is not guaranteed to be correct.",
    )
    @argument(
        "--output",
        default="parquet",
        help='Output format for the query results. One of "parquet", or "csv".',
    )
    @argument(
        "--dataframe",
        default=True,
        help="When `True`, will render the results as a dataframe. Otherwise only returns the response.",
    )
    @argument(
        "--preview-rows",
        help="Produces a preview of the data with at least this many rows.",
    )
    @argument(
        "--result-behavior",
        help='Determines which results are returned. \
            Either "all-results" (default), or "final-results" which returns only the final values for each entity.',
    )
    @argument(
        "--final-time",
        help="Produces final values for each entity at the given timestamp.",
    )
    @argument("--var", help="Assigns the body to a local variable with the given name")
    @cell_magic
    @line_cell_magic
    def fenl(self, arg, cell=None):
        "fenl query magic"

        args = parse_argstring(self.fenl, arg)
        dry_run = test_arg(clean_arg(str(args.dry_run)), "true")
        experimental = test_arg(clean_arg(str(args.experimental)), "true")
        render_dataframe = test_arg(clean_arg(str(args.dataframe)), "true")
        response_as = arg_to_response_type(clean_arg(args.output))
        result_behavior = clean_arg(args.result_behavior)
        preview_rows = clean_arg(args.preview_rows)
        var = clean_arg(args.var)
        final_result_time = clean_arg(args.final_time)

        if cell is None:
            query = arg
        else:
            query = cell.strip()

        limits = {}
        if preview_rows:
            limits["preview_rows"] = int(preview_rows)

        try:
            resp = compute.queryV2(
                query=query,
                result_behavior="final-results"
                if test_arg(result_behavior, "final-results")
                else "all-results",
                dry_run=dry_run,
                experimental=experimental,
                data_token_id=clean_arg(args.data_token),
                changed_since_time=clean_arg(args.changed_since_time),
                final_result_time=final_result_time,
                limits=query_pb.Query.Limits(**limits),
                response_as=response_as,
                client=self.client,
            )
            query_result = QueryResult(query, resp)

            if not dry_run and render_dataframe:
                # TODO: figure out how to support reading from multiple parquet paths
                if len(query_result.query_response.file_results.paths) > 0:
                    if response_as == compute.ResponseType.FILE_TYPE_PARQUET:
                        df = pandas.read_parquet(
                            query_result.query_response.file_results.paths[0],
                            engine="pyarrow",
                        )
                        query_result.set_dataframe(df)
                    elif response_as == compute.ResponseType.FILE_TYPE_CSV:
                        df = pandas.read_csv(
                            query_result.query_response.file_results.paths[0],
                        )
                        query_result.set_dataframe(df)

            if var is not None:
                IPython.get_ipython().push({var: query_result})

            return query_result
        except Exception as e:
            raise UsageError(e)


def load_ipython_extension(ipython):
    if client.KASKADA_DEFAULT_CLIENT is None:
        client_id = os.getenv("KASKADA_CLIENT_ID", None)
        client.init(client_id=client_id)

    magics = FenlMagics(ipython, client.KASKADA_DEFAULT_CLIENT)
    ipython.register_magics(magics)


def clean_arg(arg: str) -> str:
    """
    Strips quotes and double-quotes from passed arguments

    Args:
        arg (str): input argument

    Returns:
        str: cleaned output
    """
    if arg is not None:
        return arg.strip('"').strip("'")
    else:
        return None


def test_arg(arg: str, val: str) -> bool:
    """
    Compares an argument against a test value.
    Returns true when the two strings are equal in a case-insensitive and dash/underscore-insensitive manner.

    Args:
        arg (str): the argument to test
        val (str): the comparison value

    Returns:
        bool: The comparision result.
    """
    if arg is not None and val is not None:
        return arg.replace("_", "-").lower() == val.replace("_", "-").lower()
